@@ -1,6 +1,8 @@
 #ifndef NEWSBOAT_CLIARGSPARSER_H_
 #define NEWSBOAT_CLIARGSPARSER_H_
 
+#include "libnewsboat-ffi/src/cliargsparser.rs.h"
+
 #include <string>
 #include <vector>
 
@@ -10,11 +12,9 @@
 
 namespace newsboat {
 class CliArgsParser {
-	void* rs_cliargsparser = nullptr;
-
 public:
 	CliArgsParser(int argc, char* argv[]);
-	~CliArgsParser();
+	~CliArgsParser() = default;
 
 	bool do_import() const;
 
@@ -22,6 +22,8 @@ public:
 
 	bool do_vacuum() const;
 
+	bool do_cleanup() const;
+
 	std::string importfile() const;
 
 	/// If non-null, Newsboat should import read articles info from this
@@ -68,20 +70,23 @@ public:
 
 	nonstd::optional<std::string> config_file() const;
 
-	/// If non-null, Newsboat should execute these commands.
+	/// If non-empty, Newsboat should execute these commands and then quit.
 	///
 	/// \note The parser does not check if the passed commands are valid.
-	nonstd::optional<std::vector<std::string>> cmds_to_execute() const;
+	std::vector<std::string> cmds_to_execute() const;
 
 	nonstd::optional<std::string> log_file() const;
 
 	nonstd::optional<Level> log_level() const;
 
-	/// Returns the pointer to the Rust object.
+	/// Returns the reference to the Rust object.
 	///
 	/// This is only meant to be used in situations when one wants to pass
-	/// a pointer to CliArgsParser back to Rust.
-	void* get_rust_pointer() const;
+	/// a reference back to Rust.
+	const cliargsparser::bridged::CliArgsParser& get_rust_ref() const;
+
+private:
+	rust::Box<cliargsparser::bridged::CliArgsParser> rs_object;
 };
 } // namespace newsboat
 
