@@ -12,7 +12,7 @@ public:
 	~DialogsFormAction() override;
 	void prepare() override;
 	void init() override;
-	KeyMapHintEntry* get_keymap_hint() override;
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
 	std::string id() const override
 	{
 		return "dialogs";
@@ -24,7 +24,8 @@ private:
 	bool process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
-	bool update_list;
+	void update_heading();
+
 	ListWidget dialogs_list;
 };
 
