@@ -15,12 +15,25 @@ class ConfigContainer;
 class RssFeed;
 class View;
 
-struct KeyMapHintEntry {
-	Operation op;
-	char* text;
+typedef std::pair<std::string, std::string> QnaPair;
+
+enum class CommandType {
+	QUIT,
+	SAVE,
+	GOTO,
+	TAG,
+	SET,
+	SOURCE,
+	DUMPCONFIG,
+	EXEC,
+	UNKNOWN,	/// Unknown/non-existing command. Tokenized input is stored in Command.args
+	INVALID, 	/// differs from UNKNOWN in that no input was parsed
 };
 
-typedef std::pair<std::string, std::string> QnaPair;
+struct Command {
+	CommandType type;
+	std::vector<std::string> args;
+};
 
 class FormAction {
 public:
@@ -28,17 +41,21 @@ public:
 	virtual ~FormAction();
 	virtual void prepare() = 0;
 	virtual void init() = 0;
-	Stfl::Form& get_form();
 	virtual void set_redraw(bool b)
 	{
 		do_redraw = b;
 	}
 
-	virtual KeyMapHintEntry* get_keymap_hint() = 0;
+	virtual const std::vector<KeyMapHintEntry>& get_keymap_hint() const = 0;
 
 	virtual std::string id() const = 0;
 
-	virtual std::string get_value(const std::string& value);
+	std::string get_value(const std::string& name);
+	void set_value(const std::string& name, const std::string& value);
+
+	void draw_form();
+	std::string draw_form_wait_for_event(unsigned int timeout);
+	void recalculate_widget_dimensions();
 
 	virtual void handle_cmdline(const std::string& cmd);
 
@@ -50,8 +67,6 @@ public:
 
 	void start_cmdline(std::string default_value = "");
 
-	virtual void recalculate_form();
-
 	std::string get_qna_response(unsigned int i)
 	{
 		return (qna_responses.size() >= (i + 1)) ? qna_responses[i]
@@ -94,10 +109,12 @@ protected:
 
 	void start_bookmark_qna(const std::string& default_title,
 		const std::string& default_url,
-		const std::string& default_desc,
 		const std::string& default_feed_title);
-	void open_unread_items_in_browser(std::shared_ptr<RssFeed> feed,
-		bool markread);
+
+	static Command parse_command(const std::string& input,
+		std::string delimiters = " \r\n\t");
+
+	void handle_parsed_command(const Command& command);
 
 	View* v;
 	ConfigContainer* cfg;
@@ -112,8 +129,13 @@ protected:
 	std::vector<std::string> valid_cmds;
 
 private:
-	std::string prepare_keymap_hint(KeyMapHintEntry* hints);
 	void start_next_question();
+	bool handle_single_argument_set(std::string argument);
+	void handle_set(const std::vector<std::string>& args);
+	void handle_quit();
+	void handle_source(const std::vector<std::string>& args);
+	void handle_dumpconfig(const std::vector<std::string>& args);
+	void handle_exec(const std::vector<std::string>& args);
 
 	std::vector<QnaPair> qna_prompts;
 	Operation finish_operation;
