@@ -29,12 +29,6 @@ public:
 	void prepare() override;
 	void init() override;
 
-	void set_redraw(bool b) override
-	{
-		FormAction::set_redraw(b);
-		invalidate_everything();
-	}
-
 	void set_feed(std::shared_ptr<RssFeed> fd);
 
 	std::string id() const override
@@ -52,7 +46,7 @@ public:
 		pos = p;
 	}
 	std::string get_guid();
-	KeyMapHintEntry* get_keymap_hint() override;
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
 
 	bool jump_to_next_unread_item(bool start_with_first);
 	bool jump_to_previous_unread_item(bool start_with_last);
@@ -62,8 +56,6 @@ public:
 
 	void handle_cmdline(const std::string& cmd) override;
 
-	void do_update_visible_items();
-
 	void finished_qna(Operation op) override;
 
 	void set_show_searchresult(bool b)
@@ -75,14 +67,26 @@ public:
 		search_phrase = s;
 	}
 
-	void recalculate_form() override;
+	void invalidate_list()
+	{
+		invalidation_mode = InvalidationMode::COMPLETE;
+	}
+
+	void restore_selected_position();
 
 private:
 	void register_format_styles();
 
+	void do_update_visible_items();
+	void draw_items();
+
 	bool process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
+
+	bool open_position_in_browser(unsigned int pos,
+		bool interactive) const;
+
 	void set_head(const std::string& s,
 		unsigned int unread,
 		unsigned int total,
@@ -92,6 +96,8 @@ private:
 	void save_article(const std::string& filename,
 		std::shared_ptr<RssItem> item);
 
+	void handle_save(const std::vector<std::string>& cmd_args);
+
 	void save_filterpos();
 
 	void qna_end_setfilter();
@@ -104,11 +110,6 @@ private:
 
 	void prepare_set_filterpos();
 
-	void invalidate_everything()
-	{
-		invalidation_mode = InvalidationMode::COMPLETE;
-	}
-
 	void invalidate(const unsigned int invalidated_pos)
 	{
 		if (invalidation_mode == InvalidationMode::COMPLETE) {
@@ -124,6 +125,8 @@ private:
 		const std::string& itemlist_format,
 		const std::string& datetime_format);
 
+	void goto_item(const std::string& title);
+
 	unsigned int pos;
 	std::shared_ptr<RssFeed> feed;
 	bool apply_filter;
