@@ -15,7 +15,7 @@ public:
 	~SelectFormAction() override;
 	void prepare() override;
 	void init() override;
-	KeyMapHintEntry* get_keymap_hint() override;
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
 	std::string get_selected_value()
 	{
 		return value;
@@ -54,6 +54,7 @@ private:
 		const std::string& tag,
 		unsigned int pos,
 		unsigned int width);
+	void update_heading();
 
 	ListWidget tags_list;
 };
