@@ -99,14 +99,21 @@ void print_usage(const std::string& argv0, const std::string& config_path,
 			_s("<file>"),
 			_s("import list of read articles from <file>")
 		},
-		{'h', "help", "", _s("this help")}
+		{'h', "help", "", _s("this help")},
+		{'-', "cleanup", "", _s("remove unreferenced items from cache")}
 	};
 
 	std::stringstream ss;
 	for (const auto& a : args) {
-		std::string longcolumn("-");
-		longcolumn += a.name;
-		longcolumn += ", --" + a.longname;
+		std::string longcolumn;
+		if (a.name != '-') {
+			longcolumn += "-";
+			longcolumn += a.name;
+			longcolumn += ", ";
+		} else {
+			longcolumn += "    ";
+		}
+		longcolumn += "--" + a.longname;
 		longcolumn += a.params.size() > 0 ? "=" + a.params : "";
 		ss << "\t" << longcolumn;
 		for (unsigned int j = 0; j < utils::gentabs(longcolumn); j++) {
@@ -136,6 +143,13 @@ void print_usage(const std::string& argv0, const std::string& config_path,
 	print_filepath(tr_config, config_path);
 	print_filepath(tr_urls, urls_path);
 	print_filepath(tr_cache, cache_path);
+
+	std::cout << std::endl
+		<< _("Support at #newsboat at https://libera.chat or on our mailing "
+			"list https://groups.google.com/g/newsboat")
+		<< std::endl
+		<< _("For more information, check out https://newsboat.org/")
+		<< std::endl;
 }
 
 void print_version(const std::string& argv0, unsigned int level)
@@ -146,11 +160,11 @@ void print_version(const std::string& argv0, unsigned int level)
 			<< PROGRAM_URL << std::endl;
 		ss << "Copyright (C) 2006-2015 Andreas Krennmair"
 			<< std::endl;
-		ss << "Copyright (C) 2015-2020 Alexander Batischev"
+		ss << "Copyright (C) 2015-2022 Alexander Batischev"
 			<< std::endl;
 		ss << "Copyright (C) 2006-2017 Newsbeuter contributors"
 			<< std::endl;
-		ss << "Copyright (C) 2017-2020 Newsboat contributors"
+		ss << "Copyright (C) 2017-2022 Newsboat contributors"
 			<< std::endl;
 		ss << std::endl;
 
@@ -206,8 +220,11 @@ int main(int argc, char* argv[])
 	setlocale(LC_CTYPE, "");
 	setlocale(LC_MESSAGES, "");
 
-	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	// Internally, Newsboat stores all strings in UTF-8, so we require gettext
+	// to return messages in that encoding.
+	bind_textdomain_codeset(PACKAGE, "UTF-8");
 
 	rsspp::Parser::global_init();
 
@@ -239,22 +256,22 @@ int main(int argc, char* argv[])
 	try {
 		ret = c.run(args);
 	} catch (const newsboat::DbException& e) {
+		Stfl::reset();
 		std::cerr << strprintf::fmt(
-				_("Caught newsboat::DbException with "
-					"message: %s"),
+				_("Caught newsboat::DbException with message: %s"),
 				e.what())
 			<< std::endl;
 		::exit(EXIT_FAILURE);
 	} catch (const newsboat::MatcherException& e) {
+		Stfl::reset();
 		std::cerr << strprintf::fmt(
-				_("Caught newsboat::MatcherException with "
-					"message: %s"),
+				_("Caught newsboat::MatcherException with message: %s"),
 				e.what())
 			<< std::endl;
 		::exit(EXIT_FAILURE);
 	} catch (const newsboat::Exception& e) {
-		std::cerr << strprintf::fmt(_("Caught newsboat::Exception with "
-					"message: %s"),
+		Stfl::reset();
+		std::cerr << strprintf::fmt(_("Caught newsboat::Exception with message: %s"),
 				e.what())
 			<< std::endl;
 		::exit(EXIT_FAILURE);
