@@ -7,6 +7,7 @@
 #include "feed.h"
 #include "item.h"
 #include "rsspp_uris.h"
+#include "xmlutilities.h"
 
 #define RSS_1_0_NS "http://purl.org/rss/1.0/"
 #define RDF_URI "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
@@ -57,6 +58,7 @@ void Rss10Parser::parse_feed(Feed& f, xmlNode* rootNode)
 						"description",
 						RSS_1_0_NS)) {
 					it.description = get_content(itnode);
+					it.description_mime_type = "";
 				} else if (node_is(itnode, "date", DC_URI)) {
 					it.pubDate = w3cdtf_to_rfc822(
 							get_content(itnode));
