@@ -1,15 +1,12 @@
 use crate::cliargsparser::CliArgsParser;
 use crate::logger::{self, Level};
 use crate::utils;
-use dirs;
 use gettextrs::gettext;
-use libc;
 use std::fs::{self, DirBuilder};
 use std::io;
 use std::os::unix::fs::DirBuilderExt;
 use std::path::{Path, PathBuf};
 use strprintf::fmt;
-use xdg;
 
 pub const NEWSBOAT_SUBDIR_XDG: &str = "newsboat";
 pub const NEWSBOAT_CONFIG_SUBDIR: &str = ".newsboat";
@@ -79,7 +76,7 @@ impl ConfigPaths {
             using_nonstandard_configs: false,
         };
 
-        let env_home = dirs::home_dir();
+        let env_home = utils::home_dir();
         if env_home.is_none() {
             let uid = unsafe { libc::getuid() };
 
