@@ -1,17 +1,18 @@
 //! Parses filter expressions.
 
 use gettextrs::gettext;
-use lazy_static::lazy_static;
 use nom::{
     branch::alt,
     bytes::complete::{escaped, is_not, tag, take, take_while, take_while1},
     character::{is_alphanumeric, is_digit},
     combinator::{complete, map, opt, peek, recognize, value},
-    error::{context, ParseError, VerboseError, VerboseErrorKind},
+    error::{ErrorKind, ParseError},
     sequence::{delimited, separated_pair, terminated, tuple},
-    IResult, Offset,
+    IResult, Offset, Parser,
 };
-use std::collections::BTreeMap;
+use once_cell::unsync::OnceCell;
+use regex_rs::Regex;
+use std::vec::Vec;
 use strprintf::fmt;
 
 /// Operators that can be used in comparisons.
@@ -31,8 +32,72 @@ pub enum Operator {
 }
 
 /// Values that can be used on the right-hand side of comparisons.
-#[derive(Debug, Clone, PartialEq)]
-pub struct Value(pub String);
+pub struct Value {
+    literal: String,
+    regex: OnceCell<Result<Regex, String>>,
+}
+
+impl Value {
+    /// Construct a value from the parsed token.
+    fn new(literal: String) -> Self {
+        Self {
+            literal,
+            regex: OnceCell::new(),
+        }
+    }
+
+    /// Access the stored literal as a string.
+    pub fn literal(&self) -> &str {
+        &self.literal
+    }
+
+    /// The literal interpreted as a POSIX extended regular expression.
+    ///
+    /// When matching, case will be ignored, and no parenthesised sub-expressions will be
+    /// extracted.
+    ///
+    /// Returns `Ok` with a regex or an `Err` with an error message.
+    pub fn as_regex(&self) -> Result<&Regex, &str> {
+        let regex = self.regex.get_or_init(|| {
+            use regex_rs::CompFlags;
+
+            Regex::new(
+                &self.literal,
+                CompFlags::EXTENDED | CompFlags::IGNORE_CASE | CompFlags::NO_SUB,
+            )
+        });
+
+        match regex {
+            Ok(regex) => Ok(regex),
+            Err(message) => Err(message),
+        }
+    }
+}
+
+impl core::fmt::Debug for Value {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> std::result::Result<(), std::fmt::Error> {
+        f.debug_struct("Value")
+            .field("literal", &self.literal)
+            .finish()
+    }
+}
+
+impl Clone for Value {
+    fn clone(&self) -> Self {
+        Self {
+            literal: self.literal.clone(),
+            regex: OnceCell::new(),
+        }
+    }
+}
+
+impl PartialEq for Value {
+    fn eq(&self, other: &Value) -> bool {
+        self.literal == other.literal
+    }
+}
+
+impl Eq for Value {}
 
 /// Parsed filter expression.
 ///
@@ -49,6 +114,26 @@ pub enum Expression {
     },
 }
 
+/// Used to state the type of thing that the parser expects to find.
+#[derive(PartialEq, Clone, Copy, Debug)]
+enum Expected {
+    AttributeName,
+    Operators,
+    Value,
+}
+
+/// Given a Expected enum value, returns a String with a translated error message
+/// indicating the thing thah was expected.
+fn translate_expected(expected: Expected) -> String {
+    match expected {
+        Expected::AttributeName => gettext("attribute name"),
+        // Don't translate "between" -- it's a keyword, not an English word.
+        Expected::Operators => gettext("one of: =~, ==, =, !~, !=, <=, >=, <, >, between, #, !#"),
+        // The options ("quoted string" etc.) are not keywords, so please translate them.
+        Expected::Value => gettext("one of: quoted string, range, number"),
+    }
+}
+
 /// Errors that may come up during parsing.
 #[derive(PartialEq, Debug)]
 enum Error<'a> {
@@ -56,49 +141,80 @@ enum Error<'a> {
     /// given position).
     TrailingCharacters(usize, &'a str),
 
-    /// Parsing error at given position. "Explanation string" is one of `EXPECTED_*` constants.
-    AtPos(usize, &'static str),
+    /// Parsing error at given position, where a value of given type was expected.
+    AtPos(usize, Expected),
 
     /// Parse error that has no explanations attached to it.
     Internal,
 }
 
-// These strings are used as identifiers in nom::context macro, which unfortunately requires
-// &'static str. The identifiers are then turned into actual, internationalized messages by
-// expected_to_i18n_msg().
-static EXPECTED_ATTRIBUTE_NAME: &str = "EXPECTED_ATTRIBUTE_NAME";
-static EXPECTED_OPERATORS: &str = "EXPECTED_OPERATORS";
-static EXPECTED_VALUE: &str = "EXPECTED_VALUE";
-
-fn expected_to_i18n_msg(expected_id: &'static str) -> &'static str {
-    lazy_static! {
-        static ref ID_TO_I18N: BTreeMap<&'static str, String> = {
-            let mut result = BTreeMap::new();
-            result.insert(EXPECTED_ATTRIBUTE_NAME, gettext("attribute name"));
-            result.insert(
-                EXPECTED_OPERATORS,
-                gettext("one of: =~, ==, =, !~, !=, <=, >=, <, >, between, #, !#"),
-            );
-            result.insert(
-                EXPECTED_VALUE,
-                gettext("one of: quoted string, range, number"),
-            );
-            result
-        };
-    }
-
-    ID_TO_I18N
-        .get(expected_id)
-        .map(|s| s.as_str())
-        // This message is intentionally left untranslated, to make it more eye-catching for
-        // non-English-speaking users. If this ever pops up, it's be nice to hear of it ASAP, so it
-        // pays to make it look unusual.
-        .unwrap_or("<internal error in filterparser::expected_to_i18n_msg>")
+/// A custom error type for the parser.
+#[derive(PartialEq, Debug)]
+struct FilterParserError<'a> {
+    pub errors: Vec<(&'a str, FilterParserErrorKind)>,
 }
 
-fn operators<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Operator, E> {
-    context(
-        EXPECTED_OPERATORS,
+/// Kind of error
+#[derive(PartialEq, Debug)]
+enum FilterParserErrorKind {
+    Nom(ErrorKind),
+    Unexpected(Expected),
+}
+
+/// ParseError implementation for the custom error type.
+impl<'a> ParseError<&'a str> for FilterParserError<'a> {
+    fn from_error_kind(input: &'a str, kind: ErrorKind) -> Self {
+        FilterParserError {
+            errors: vec![(input, FilterParserErrorKind::Nom(kind))],
+        }
+    }
+
+    fn append(input: &'a str, kind: ErrorKind, mut other: Self) -> Self {
+        other.errors.push((input, FilterParserErrorKind::Nom(kind)));
+        other
+    }
+}
+
+/// Trait for adding a expected type context. Resembles the nom::error::ContextError trait.
+trait ExpectativeError<I>: Sized {
+    fn add_expectative(input: I, expected: Expected, other: Self) -> Self;
+}
+
+/// Implement ExpectativeError for FilterParserError.
+impl<'a> ExpectativeError<&'a str> for FilterParserError<'a> {
+    fn add_expectative(input: &'a str, expected: Expected, mut other: Self) -> Self {
+        other
+            .errors
+            .push((input, FilterParserErrorKind::Unexpected(expected)));
+        other
+    }
+}
+
+/// Create a new error from an input position, a expected kind of value and an existing error.
+/// This combinator is used to add the possibility of adding translated user friendly information
+/// to errors when backtracking through a parse tree.
+fn expect<I: Clone, E: ExpectativeError<I>, F, O>(
+    expected: Expected,
+    mut f: F,
+) -> impl FnMut(I) -> IResult<I, O, E>
+where
+    F: Parser<I, O, E>,
+{
+    move |i: I| match f.parse(i.clone()) {
+        Ok(o) => Ok(o),
+        Err(nom::Err::Incomplete(i)) => Err(nom::Err::Incomplete(i)),
+        Err(nom::Err::Error(e)) => Err(nom::Err::Error(E::add_expectative(i, expected, e))),
+        Err(nom::Err::Failure(e)) => Err(nom::Err::Failure(E::add_expectative(i, expected, e))),
+    }
+}
+
+fn operators<'a, E: ParseError<&'a str> + ExpectativeError<&'a str>>(
+    input: &'a str,
+) -> IResult<&'a str, Operator, E> {
+    // State the expected kind of value (operator), so we can see an
+    // especific error message when this parser fails.
+    expect(
+        Expected::Operators,
         alt((
             value(Operator::RegexMatches, tag("=~")),
             value(Operator::Equals, alt((tag("=="), tag("=")))),
@@ -126,7 +242,7 @@ fn quoted_string<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str,
         Ok((leftovers, String::from(chr)))
     };
 
-    map(alt((nonempty_string, empty_string)), Value)(input)
+    map(alt((nonempty_string, empty_string)), Value::new)(input)
 }
 
 fn number<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, &'a str, E> {
@@ -135,7 +251,7 @@ fn number<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, &'a st
 
 fn range<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Value, E> {
     separated_pair(number, tag(":"), number)(input)
-        .map(|(leftovers, (a, b))| (leftovers, Value(format!("{}:{}", a, b))))
+        .map(|(leftovers, (a, b))| (leftovers, Value::new(format!("{}:{}", a, b))))
 }
 
 /// Skips zero or more space characters.
@@ -154,9 +270,13 @@ fn space1<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, &'a st
     take_while1(|c| c == ' ')(input)
 }
 
-fn comparison<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Expression, E> {
-    let attribute_name = context(
-        EXPECTED_ATTRIBUTE_NAME,
+fn comparison<'a, E: ParseError<&'a str> + ExpectativeError<&'a str>>(
+    input: &'a str,
+) -> IResult<&'a str, Expression, E> {
+    // State the expected kind of value (attribute name), so we can see an
+    // especific error message when this parser fails.
+    let mut attribute_name = expect(
+        Expected::AttributeName,
         take_while1(|c| is_alphanumeric(c as u8) || c == '_' || c == '-' || c == '.'),
     );
 
@@ -165,9 +285,15 @@ fn comparison<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Ex
     let (input, _) = space0(input)?;
     let (input, op) = operators(input)?;
     let (input, _) = space0(input)?;
-    let (leftovers, value) = context(
-        EXPECTED_VALUE,
-        alt((quoted_string, range, map(number, |n| Value(n.to_string())))),
+    // State the expected kind of value (value), so we can see an
+    // especific error message when this parser fails.
+    let (leftovers, value) = expect(
+        Expected::Value,
+        alt((
+            quoted_string,
+            range,
+            map(number, |n| Value::new(n.to_string())),
+        )),
     )(input)?;
 
     Ok((
@@ -180,7 +306,9 @@ fn comparison<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Ex
     ))
 }
 
-fn parens<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Expression, E> {
+fn parens<'a, E: ParseError<&'a str> + ExpectativeError<&'a str>>(
+    input: &'a str,
+) -> IResult<&'a str, Expression, E> {
     let (input, _) = tag("(")(input)?;
     let (input, _) = space0(input)?;
     let (input, result) = alt((expression, parens, comparison))(input)?;
@@ -208,7 +336,9 @@ fn space_after_logop<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a
     peek(parser)(input)
 }
 
-fn expression<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Expression, E> {
+fn expression<'a, E: ParseError<&'a str> + ExpectativeError<&'a str>>(
+    input: &'a str,
+) -> IResult<&'a str, Expression, E> {
     // `Expression`s enum variants can't be used as return values without filling in their
     // arguments, so we have to create another enum, which variants we can return and `match` on.
     // This is better than matching on bare strings returned by `tag`, as it enables us to write an
@@ -218,7 +348,7 @@ fn expression<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Ex
     enum Op {
         And,
         Or,
-    };
+    }
 
     let (input, left) = alt((parens, comparison))(input)?;
     let (input, _) = space0(input)?;
@@ -237,9 +367,11 @@ fn expression<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Ex
     Ok((leftovers, op))
 }
 
-fn parser<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Expression, E> {
+fn parser<'a, E: ParseError<&'a str> + ExpectativeError<&'a str>>(
+    input: &'a str,
+) -> IResult<&'a str, Expression, E> {
     let parsers = alt((expression, parens, comparison));
-    // Ignore leading and trailing whitespace
+    // Ignore leading and trailing whitespace.
     let parsers = delimited(space0, parsers, space0);
     // Try to parse input. If parser says it needs more data, make that an error, since `input` is
     // all we got.
@@ -247,23 +379,22 @@ fn parser<'a, E: ParseError<&'a str>>(input: &'a str) -> IResult<&'a str, Expres
 }
 
 fn internal_parse(expr: &str) -> Result<Expression, Error> {
-    match parser::<VerboseError<&str>>(expr) {
+    match parser::<FilterParserError>(expr) {
         Ok((leftovers, expression)) => {
             if leftovers.is_empty() {
                 Ok(expression)
             } else {
-                Err(Error::TrailingCharacters(
-                    expr.offset(leftovers),
-                    &leftovers,
-                ))
+                Err(Error::TrailingCharacters(expr.offset(leftovers), leftovers))
             }
         }
         Err(error) => {
-            let handler = |e: VerboseError<&str>| -> Error {
+            let handler = |e: FilterParserError| -> Error {
                 for (chunk, err) in e.errors {
                     let pos = expr.offset(chunk);
                     match err {
-                        VerboseErrorKind::Context(expected) => return Error::AtPos(pos, expected),
+                        FilterParserErrorKind::Unexpected(expected) => {
+                            return Error::AtPos(pos, expected)
+                        }
                         _ => continue,
                     }
                 }
@@ -290,18 +421,18 @@ pub fn parse(expr: &str) -> Result<Expression, String> {
         Err(error) => {
             let err = match error {
                 Error::TrailingCharacters(pos, tail) => fmt!(
-                    // The "%{}" thing is a number, a zero-based offset into a string.
-                    &gettext("Parse error: trailing characters after position %{}: %s"),
-                    PRIu64,
-                    pos as u64,
+                    // The first %s is an integer offset at which trailing characters start, the
+                    // second %s is the tail itself.
+                    &gettext("Parse error: trailing characters after position %s: %s"),
+                    &pos.to_string(),
                     tail
                 ),
                 Error::AtPos(pos, expected) => fmt!(
-                    // The "%{}" thing is a number, a zero-based offset into a string.
-                    &gettext("Parse error at position %{}: expected %s"),
-                    PRIu64,
-                    pos as u64,
-                    expected_to_i18n_msg(expected)
+                    // The first %s is a zero-based offset into the string, the second %s is the
+                    // description of what the program expected at that point.
+                    &gettext("Parse error at position %s: expected %s"),
+                    &pos.to_string(),
+                    &translate_expected(expected)
                 ),
                 Error::Internal => fmt!(&gettext("Internal parse error")),
             };
@@ -319,22 +450,22 @@ mod tests {
         // Invalid character in operator
         assert_eq!(
             internal_parse("title =¯ \"foo\""),
-            Err(Error::AtPos(7, EXPECTED_VALUE))
+            Err(Error::AtPos(7, Expected::Value))
         );
         // Incorrect string quoting
         assert_eq!(
             internal_parse("a = \"b"),
-            Err(Error::AtPos(4, EXPECTED_VALUE))
+            Err(Error::AtPos(4, Expected::Value))
         );
         // Non-value to the right of equality operator
         assert_eq!(
             internal_parse("a = b"),
-            Err(Error::AtPos(4, EXPECTED_VALUE))
+            Err(Error::AtPos(4, Expected::Value))
         );
         // Non-existent operator
         assert_eq!(
             internal_parse("a !! \"b\""),
-            Err(Error::AtPos(2, EXPECTED_OPERATORS))
+            Err(Error::AtPos(2, Expected::Operators))
         );
 
         // Unbalanced parentheses
@@ -360,7 +491,7 @@ mod tests {
         // Operator without arguments
         assert_eq!(
             internal_parse("=!"),
-            Err(Error::AtPos(0, EXPECTED_ATTRIBUTE_NAME))
+            Err(Error::AtPos(0, Expected::AttributeName))
         );
     }
 
@@ -381,7 +512,7 @@ mod tests {
         let expected = Ok(Expression::Comparison {
             attribute: "a".to_string(),
             op: Operator::Equals,
-            value: Value("abc".to_string()),
+            value: Value::new("abc".to_string()),
         });
 
         assert_eq!(internal_parse("a = \"abc\""), expected);
@@ -404,7 +535,7 @@ mod tests {
             Ok(Expression::Comparison {
                 attribute: "attribute".to_string(),
                 op: Operator::Equals,
-                value: Value("hello\0world".to_string()),
+                value: Value::new("hello\0world".to_string()),
             })
         );
     }
@@ -414,7 +545,7 @@ mod tests {
         let expected = Ok(Expression::Comparison {
             attribute: "title".to_string(),
             op: Operator::Equals,
-            value: Value(String::new()),
+            value: Value::new(String::new()),
         });
 
         assert_eq!(internal_parse("title==\"\""), expected);
@@ -430,12 +561,12 @@ mod tests {
             Box::new(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::Equals,
-                value: Value("42".to_string()),
+                value: Value::new("42".to_string()),
             }),
             Box::new(Comparison {
                 attribute: "y".to_string(),
                 op: Operator::Equals,
-                value: Value("0".to_string()),
+                value: Value::new("0".to_string()),
             }),
         );
 
@@ -454,12 +585,12 @@ mod tests {
             Box::new(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::Equals,
-                value: Value("42".to_string()),
+                value: Value::new("42".to_string()),
             }),
             Box::new(Comparison {
                 attribute: "y".to_string(),
                 op: Operator::Equals,
-                value: Value("0".to_string()),
+                value: Value::new("0".to_string()),
             }),
         );
 
@@ -473,7 +604,7 @@ mod tests {
         let expected = Comparison {
             attribute: "array".to_string(),
             op: Operator::Contains,
-            value: Value("bar".to_string()),
+            value: Value::new("bar".to_string()),
         };
 
         assert_eq!(internal_parse("array # \"bar\""), Ok(expected.clone()));
@@ -492,19 +623,19 @@ mod tests {
     fn t_only_space_characters_are_considered_whitespace_by_filter_parser() {
         assert_eq!(
             internal_parse("attr\t= \"value\""),
-            Err(Error::AtPos(4, EXPECTED_OPERATORS))
+            Err(Error::AtPos(4, Expected::Operators))
         );
         assert_eq!(
             internal_parse("attr =\t\"value\""),
-            Err(Error::AtPos(6, EXPECTED_VALUE))
+            Err(Error::AtPos(6, Expected::Value))
         );
         assert_eq!(
             internal_parse("attr\n=\t\"value\""),
-            Err(Error::AtPos(4, EXPECTED_OPERATORS))
+            Err(Error::AtPos(4, Expected::Operators))
         );
         assert_eq!(
             internal_parse("attr\u{b}=\"value\""),
-            Err(Error::AtPos(4, EXPECTED_OPERATORS))
+            Err(Error::AtPos(4, Expected::Operators))
         );
         assert_eq!(
             internal_parse("attr=\"value\"\r\n"),
@@ -520,12 +651,12 @@ mod tests {
                 Box::new(Comparison {
                     attribute: "x".to_string(),
                     op: Operator::Equals,
-                    value: Value("42".to_string())
+                    value: Value::new("42".to_string())
                 }),
                 Box::new(Comparison {
                     attribute: "y".to_string(),
                     op: Operator::Equals,
-                    value: Value("0".to_string())
+                    value: Value::new("0".to_string())
                 })
             ))
         );
@@ -536,12 +667,12 @@ mod tests {
                 Box::new(Comparison {
                     attribute: "x".to_string(),
                     op: Operator::Equals,
-                    value: Value("42".to_string())
+                    value: Value::new("42".to_string())
                 }),
                 Box::new(Comparison {
                     attribute: "y".to_string(),
                     op: Operator::Equals,
-                    value: Value("0".to_string())
+                    value: Value::new("0".to_string())
                 })
             ))
         );
@@ -552,12 +683,12 @@ mod tests {
                 Box::new(Comparison {
                     attribute: "x".to_string(),
                     op: Operator::Equals,
-                    value: Value("42".to_string())
+                    value: Value::new("42".to_string())
                 }),
                 Box::new(Comparison {
                     attribute: "y".to_string(),
                     op: Operator::Equals,
-                    value: Value("42".to_string())
+                    value: Value::new("42".to_string())
                 })
             ))
         );
@@ -572,7 +703,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::Equals,
-                value: Value("b".to_string())
+                value: Value::new("b".to_string())
             })
         );
 
@@ -581,7 +712,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::NotEquals,
-                value: Value("b".to_string())
+                value: Value::new("b".to_string())
             })
         );
 
@@ -590,7 +721,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::RegexMatches,
-                value: Value("b".to_string())
+                value: Value::new("b".to_string())
             })
         );
 
@@ -599,7 +730,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::NotRegexMatches,
-                value: Value("b".to_string())
+                value: Value::new("b".to_string())
             })
         );
 
@@ -610,7 +741,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::LessThan,
-                value: Value("b".to_string())
+                value: Value::new("b".to_string())
             })
         );
 
@@ -619,7 +750,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::LessThanOrEquals,
-                value: Value("b".to_string())
+                value: Value::new("b".to_string())
             })
         );
 
@@ -628,7 +759,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::GreaterThan,
-                value: Value("abc".to_string())
+                value: Value::new("abc".to_string())
             })
         );
 
@@ -637,7 +768,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::Equals,
-                value: Value("abc".to_string())
+                value: Value::new("abc".to_string())
             })
         );
 
@@ -646,7 +777,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "a".to_string(),
                 op: Operator::GreaterThanOrEquals,
-                value: Value("3".to_string())
+                value: Value::new("3".to_string())
             })
         );
 
@@ -655,7 +786,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "some_value".to_string(),
                 op: Operator::Between,
-                value: Value("0:-1".to_string())
+                value: Value::new("0:-1".to_string())
             })
         );
 
@@ -664,7 +795,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "other_string".to_string(),
                 op: Operator::Between,
-                value: Value("impossible".to_string())
+                value: Value::new("impossible".to_string())
             })
         );
 
@@ -673,7 +804,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "array".to_string(),
                 op: Operator::Contains,
-                value: Value("name".to_string())
+                value: Value::new("name".to_string())
             })
         );
 
@@ -682,7 +813,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "answers".to_string(),
                 op: Operator::NotContains,
-                value: Value("42".to_string())
+                value: Value::new("42".to_string())
             })
         );
 
@@ -691,7 +822,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "author".to_string(),
                 op: Operator::RegexMatches,
-                value: Value("\\s*Doe$".to_string())
+                value: Value::new("\\s*Doe$".to_string())
             })
         );
     }
@@ -704,18 +835,18 @@ mod tests {
                 Box::new(Comparison {
                     attribute: "a".to_string(),
                     op: Operator::Equals,
-                    value: Value("b".to_string())
+                    value: Value::new("b".to_string())
                 }),
                 Box::new(Or(
                     Box::new(Comparison {
                         attribute: "b".to_string(),
                         op: Operator::Equals,
-                        value: Value("c".to_string())
+                        value: Value::new("c".to_string())
                     }),
                     Box::new(Comparison {
                         attribute: "c".to_string(),
                         op: Operator::Equals,
-                        value: Value("d".to_string())
+                        value: Value::new("d".to_string())
                     }),
                 ))
             )
@@ -727,18 +858,18 @@ mod tests {
                 Box::new(Comparison {
                     attribute: "a".to_string(),
                     op: Operator::Equals,
-                    value: Value("b".to_string())
+                    value: Value::new("b".to_string())
                 }),
                 Box::new(And(
                     Box::new(Comparison {
                         attribute: "b".to_string(),
                         op: Operator::Equals,
-                        value: Value("c".to_string())
+                        value: Value::new("c".to_string())
                     }),
                     Box::new(Comparison {
                         attribute: "c".to_string(),
                         op: Operator::Equals,
-                        value: Value("d".to_string())
+                        value: Value::new("d".to_string())
                     }),
                 ))
             )
@@ -751,18 +882,18 @@ mod tests {
                     Box::new(Comparison {
                         attribute: "a".to_string(),
                         op: Operator::Equals,
-                        value: Value("b".to_string())
+                        value: Value::new("b".to_string())
                     }),
                     Box::new(Comparison {
                         attribute: "b".to_string(),
                         op: Operator::Equals,
-                        value: Value("c".to_string())
+                        value: Value::new("c".to_string())
                     }),
                 )),
                 Box::new(Comparison {
                     attribute: "c".to_string(),
                     op: Operator::Equals,
-                    value: Value("d".to_string())
+                    value: Value::new("d".to_string())
                 })
             )
         );
@@ -779,7 +910,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "value".to_string(),
                 op: Operator::Between,
-                value: Value("-100:-1".to_string())
+                value: Value::new("-100:-1".to_string())
             })
         );
 
@@ -788,7 +919,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "value".to_string(),
                 op: Operator::Between,
-                value: Value("-100:100500".to_string())
+                value: Value::new("-100:100500".to_string())
             })
         );
 
@@ -797,7 +928,7 @@ mod tests {
             Ok(Comparison {
                 attribute: "value".to_string(),
                 op: Operator::Between,
-                value: Value("123:-10".to_string())
+                value: Value::new("123:-10".to_string())
             })
         );
     }
@@ -806,17 +937,17 @@ mod tests {
         #[test]
         fn does_not_crash_on_any_input(ref input in "\\PC*") {
             // Result explicitly ignored because we just want to make sure this call doesn't panic.
-            let _ = internal_parse(&input);
+            let _ = internal_parse(input);
         }
 
         #[test]
         fn whitespace_doesnt_affect_results_1(ref input in r#" *a *!= *"b" *"#) {
             assert_eq!(
-                internal_parse(&input),
+                internal_parse(input),
                 Ok(Comparison {
                     attribute: "a".to_string(),
                     op: Operator::NotEquals,
-                    value: Value("b".to_string())
+                    value: Value::new("b".to_string())
                 })
             );
         }
@@ -824,11 +955,11 @@ mod tests {
         #[test]
         fn whitespace_doesnt_affect_results_2(ref input in r#" *( *a *!= *"b" *) *"#) {
             assert_eq!(
-                internal_parse(&input),
+                internal_parse(input),
                 Ok(Comparison {
                     attribute: "a".to_string(),
                     op: Operator::NotEquals,
-                    value: Value("b".to_string())
+                    value: Value::new("b".to_string())
                 })
             );
         }
@@ -836,7 +967,7 @@ mod tests {
         #[test]
         fn attribute_names_can_contain_alphanumerics_underscore_dash_and_dot(ref input in r#"[-A-Za-z0-9_.]+ == 0"#) {
             assert!(
-                internal_parse(&input).is_ok(),
+                internal_parse(input).is_ok(),
             );
         }
 
@@ -844,7 +975,7 @@ mod tests {
         fn no_internal_parsing_errors(ref input in "\\PC*") {
             // We should return either a parsed expression or a descriptive error -- never
             // a nondescript "internal error".
-            assert_ne!(internal_parse(&input), Err(Error::Internal));
+            assert_ne!(internal_parse(input), Err(Error::Internal));
         }
     }
 }
