@@ -3,7 +3,6 @@
 use crate::filterparser::{self, Expression, Expression::*, Operator, Value};
 use crate::matchable::Matchable;
 use crate::matchererror::MatcherError;
-use regex_rs::{CompFlags, MatchFlags, Regex};
 
 /// Checks if given filter expression is true for a given feed or article.
 ///
@@ -43,16 +42,15 @@ impl Matcher {
 impl Operator {
     fn apply(&self, attr: &str, value: &Value) -> Result<bool, MatcherError> {
         match self {
-            Operator::Equals => Ok(attr == value.0),
+            Operator::Equals => Ok(attr == value.literal()),
             Operator::NotEquals => Operator::Equals.apply(attr, value).map(|result| !result),
-            Operator::RegexMatches => match Regex::new(
-                &value.0,
-                CompFlags::EXTENDED | CompFlags::IGNORE_CASE | CompFlags::NO_SUB,
-            ) {
+            Operator::RegexMatches => match value.as_regex() {
                 Ok(regex) => {
                     let mut result = false;
                     let max_matches = 1;
-                    if let Ok(matches) = regex.matches(attr, max_matches, MatchFlags::empty()) {
+                    if let Ok(matches) =
+                        regex.matches(attr, max_matches, regex_rs::MatchFlags::empty())
+                    {
                         // Ok with non-empty Vec inside means a match was found
                         result = !matches.is_empty();
                     }
@@ -60,19 +58,21 @@ impl Operator {
                 }
 
                 Err(errmsg) => Err(MatcherError::InvalidRegex {
-                    regex: value.0.clone(),
-                    errmsg,
+                    regex: value.literal().to_string(),
+                    errmsg: errmsg.to_string(),
                 }),
             },
             Operator::NotRegexMatches => Operator::RegexMatches
                 .apply(attr, value)
                 .map(|result| !result),
-            Operator::LessThan => Ok(string_to_num(attr) < string_to_num(&value.0)),
-            Operator::GreaterThan => Ok(dbg!(string_to_num(attr)) > dbg!(string_to_num(&value.0))),
-            Operator::LessThanOrEquals => Ok(string_to_num(attr) <= string_to_num(&value.0)),
-            Operator::GreaterThanOrEquals => Ok(string_to_num(attr) >= string_to_num(&value.0)),
+            Operator::LessThan => Ok(string_to_num(attr) < string_to_num(value.literal())),
+            Operator::GreaterThan => Ok(string_to_num(attr) > string_to_num(value.literal())),
+            Operator::LessThanOrEquals => Ok(string_to_num(attr) <= string_to_num(value.literal())),
+            Operator::GreaterThanOrEquals => {
+                Ok(string_to_num(attr) >= string_to_num(value.literal()))
+            }
             Operator::Between => {
-                let fields = value.0.split(':').collect::<Vec<_>>();
+                let fields = value.literal().split(':').collect::<Vec<_>>();
                 if fields.len() != 2 {
                     return Ok(false);
                 }
@@ -87,7 +87,7 @@ impl Operator {
             }
             Operator::Contains => {
                 for token in attr.split(' ') {
-                    if token == value.0 {
+                    if token == value.literal() {
                         return Ok(true);
                     }
                 }
@@ -107,7 +107,7 @@ fn string_to_num(input: &str) -> i32 {
 
     let numerics_end = input[search_start..]
         .find(|c: char| !c.is_numeric())
-        .unwrap_or_else(|| input.len())
+        .unwrap_or(input.len())
         + search_start; // Adding the starting offset to get an index inside the original input
 
     if numerics_end - search_start == 0 {
@@ -132,12 +132,12 @@ fn evaluate_expression(expr: &Expression, item: &impl Matchable) -> Result<bool,
             attribute,
             op,
             value,
-        } => match item.attribute_value(&attribute) {
+        } => match item.attribute_value(attribute) {
             None => Err(MatcherError::AttributeUnavailable {
                 attr: attribute.clone(),
             }),
 
-            Some(ref attr) => op.apply(attr, &value),
+            Some(ref attr) => op.apply(attr, value),
         },
         And(left, right) => evaluate_expression(left, item).and_then(|result| {
             if result {
@@ -381,7 +381,7 @@ mod tests {
             match Matcher::parse(expression).unwrap().matches(&mock) {
                 Err(MatcherError::AttributeUnavailable { .. }) => { /* that's the expected result */
                 }
-                result => panic!(format!("unexpected result: {:?}", result)),
+                result => panic!("unexpected result: {:?}", result),
             }
         };
 
@@ -399,12 +399,12 @@ mod tests {
 
         match Matcher::parse("AAAA =~ \"[[\"").unwrap().matches(&mock) {
             Err(MatcherError::InvalidRegex { .. }) => { /* that's the expected result */ }
-            result => panic!(format!("unexpected result: {:?}", result)),
+            result => panic!("unexpected result: {:?}", result),
         }
 
         match Matcher::parse("AAAA !~ \"[[\"").unwrap().matches(&mock) {
             Err(MatcherError::InvalidRegex { .. }) => { /* that's the expected result */ }
-            result => panic!(format!("unexpected result: {:?}", result)),
+            result => panic!("unexpected result: {:?}", result),
         }
     }
 
@@ -987,6 +987,9 @@ mod tests {
     // ensure that we're using EREs, these tests try stuff that's *not*
     // supported by EREs.
     //
+    // FreeBSD 13 returns an error from regexec() instead of failing the match like other OSes do.
+    // That's why we do the `match` instead of simply chaining the results with `unwrap()`.
+    //
     // Ideas gleaned from https://www.regular-expressions.info/refcharacters.html
 
     #[test]
@@ -995,56 +998,72 @@ mod tests {
 
         let mock = MockMatchable::new(&[("attr", "*]+")]);
 
-        assert!(!Matcher::parse(r#"(attr =~ "\Q*]+\E")"#)
+        match Matcher::parse(r#"(attr =~ "\Q*]+\E")"#)
             .unwrap()
             .matches(&mock)
-            .unwrap());
-        assert!(Matcher::parse(r#"(attr !~ "\Q*]+\E")"#)
+        {
+            Ok(result) => assert!(!result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
+
+        match Matcher::parse(r#"(attr !~ "\Q*]+\E")"#)
             .unwrap()
             .matches(&mock)
-            .unwrap());
+        {
+            Ok(result) => assert!(result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
     }
 
     #[test]
     fn t_regex_matching_doesnt_support_hexadecimal_escape() {
         let mock = MockMatchable::new(&[("attr", "value")]);
 
-        assert!(!Matcher::parse(r#"(attr =~ "^va\x6Cue")"#)
+        match Matcher::parse(r#"(attr =~ "^va\x6Cue")"#)
             .unwrap()
             .matches(&mock)
-            .unwrap());
-        assert!(Matcher::parse(r#"attr !~ "^va\x6Cue""#)
+        {
+            Ok(result) => assert!(!result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
+
+        match Matcher::parse(r#"attr !~ "^va\x6Cue""#)
             .unwrap()
             .matches(&mock)
-            .unwrap());
+        {
+            Ok(result) => assert!(result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
     }
 
     #[test]
     fn t_regex_matching_doesnt_support_backslash_a_as_alert_control_character() {
         let mock = MockMatchable::new(&[("attr", "\x07")]);
 
-        assert!(!Matcher::parse(r#"(attr =~ "\a")"#)
-            .unwrap()
-            .matches(&mock)
-            .unwrap());
-        assert!(Matcher::parse(r#"(attr !~ "\a")"#)
-            .unwrap()
-            .matches(&mock)
-            .unwrap());
+        match Matcher::parse(r#"(attr =~ "\a")"#).unwrap().matches(&mock) {
+            Ok(result) => assert!(!result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
+
+        match Matcher::parse(r#"(attr !~ "\a")"#).unwrap().matches(&mock) {
+            Ok(result) => assert!(result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
     }
 
     #[test]
     fn t_regex_matching_doesnt_support_backslash_b_as_backspace_control_character() {
         let mock = MockMatchable::new(&[("attr", "\x08")]);
 
-        assert!(!Matcher::parse(r#"(attr =~ "\b")"#)
-            .unwrap()
-            .matches(&mock)
-            .unwrap());
-        assert!(Matcher::parse(r#"(attr !~ "\b")"#)
-            .unwrap()
-            .matches(&mock)
-            .unwrap());
+        match Matcher::parse(r#"(attr =~ "\b")"#).unwrap().matches(&mock) {
+            Ok(result) => assert!(!result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
+
+        match Matcher::parse(r#"(attr !~ "\b")"#).unwrap().matches(&mock) {
+            Ok(result) => assert!(result),
+            Err(e) => assert!(matches!(e, MatcherError::InvalidRegex { .. })),
+        }
     }
 
     #[test]
