@@ -44,6 +44,9 @@ ConfigContainer::ConfigContainer()
 			ConfigDataType::PATH)},
 	{"cache-file", ConfigData("", ConfigDataType::PATH)},
 	{"cleanup-on-quit", ConfigData("yes", ConfigDataType::BOOL)},
+	{"confirm-delete-all-articles", ConfigData("yes", ConfigDataType::BOOL)},
+	{"confirm-mark-all-feeds-read", ConfigData("yes", ConfigDataType::BOOL)},
+	{"confirm-mark-feed-read", ConfigData("yes", ConfigDataType::BOOL)},
 	{"confirm-exit", ConfigData("no", ConfigDataType::BOOL)},
 	{"cookie-cache", ConfigData("", ConfigDataType::PATH)},
 	{"datetime-format", ConfigData("%b %d", ConfigDataType::STR)},
@@ -82,6 +85,18 @@ ConfigContainer::ConfigContainer()
 	{
 		"feedhq-url",
 		ConfigData("https://feedhq.org/", ConfigDataType::STR)},
+	{"freshrss-flag-star", ConfigData("", ConfigDataType::STR)},
+	{"freshrss-login", ConfigData("", ConfigDataType::STR)},
+	{"freshrss-min-items", ConfigData("20", ConfigDataType::INT)},
+	{"freshrss-password", ConfigData("", ConfigDataType::STR)},
+	{"freshrss-passwordfile", ConfigData("", ConfigDataType::PATH)},
+	{"freshrss-passwordeval", ConfigData("", ConfigDataType::STR)},
+	{
+		"freshrss-show-special-feeds",
+		ConfigData("true", ConfigDataType::BOOL)},
+	{
+		"freshrss-url",
+		ConfigData("", ConfigDataType::STR)},
 	{
 		"feedlist-format",
 		ConfigData("%4i %n %11u %t", ConfigDataType::STR)},
@@ -142,7 +157,7 @@ ConfigContainer::ConfigContainer()
 	{"notify-beep", ConfigData("no", ConfigDataType::BOOL)},
 	{
 		"notify-format",
-		ConfigData(_("newsboat: finished reload, %f unread "
+		ConfigData(_("Newsboat: finished reload, %f unread "
 				"feeds (%n unread articles total)"),
 			ConfigDataType::STR)},
 	{"notify-program", ConfigData("", ConfigDataType::PATH)},
@@ -203,6 +218,7 @@ ConfigContainer::ConfigContainer()
 		ConfigData("false", ConfigDataType::BOOL)},
 	{"reload-threads", ConfigData("1", ConfigDataType::INT)},
 	{"reload-time", ConfigData("60", ConfigDataType::INT)},
+	{"restrict-filename", ConfigData("yes", ConfigDataType::BOOL)},
 	{"save-path", ConfigData("~/", ConfigDataType::PATH)},
 	{"scrolloff", ConfigData("0", ConfigDataType::INT)},
 	{
@@ -246,6 +262,7 @@ ConfigContainer::ConfigContainer()
 	{"ocnews-flag-star", ConfigData("", ConfigDataType::STR)},
 	{"ocnews-url", ConfigData("", ConfigDataType::STR)},
 	{"miniflux-login", ConfigData("", ConfigDataType::STR)},
+	{"miniflux-min-items", ConfigData("100", ConfigDataType::INT)},
 	{"miniflux-password", ConfigData("", ConfigDataType::STR)},
 	{"miniflux-passwordfile", ConfigData("", ConfigDataType::PATH)},
 	{"miniflux-passwordeval", ConfigData("", ConfigDataType::STR)},
@@ -259,6 +276,7 @@ ConfigContainer::ConfigContainer()
 			"ttrss",
 			"newsblur",
 			"feedhq",
+			"freshrss",
 			"ocnews",
 			"miniflux",
 			"inoreader"}))},
@@ -268,18 +286,14 @@ ConfigContainer::ConfigContainer()
 	/* title formats: */
 	{
 		"articlelist-title-format",
-		ConfigData(_("%N %V - Articles in feed '%T' (%u unread, %t total)"
-				"%?F? matching filter `%F'&?"
-				" - %U"),
+		ConfigData(_("%N %V - Articles in feed '%T' (%u unread, %t total)%?F? matching filter '%F'&? - %U"),
 			ConfigDataType::STR)},
 	{
 		"dialogs-title-format",
 		ConfigData(_("%N %V - Dialogs"), ConfigDataType::STR)},
 	{
 		"feedlist-title-format",
-		ConfigData(_("%N %V - %?F?Feeds&Your feeds? (%u unread, %t total)"
-				"%?F? matching filter `%F'&?"
-				"%?T? - tag `%T'&?"),
+		ConfigData(_("%N %V - %?F?Feeds&Your feeds? (%u unread, %t total)%?F? matching filter '%F'&?%?T? - tag '%T'&?"),
 			ConfigDataType::STR)},
 	{
 		"filebrowser-title-format",
@@ -294,13 +308,11 @@ ConfigContainer::ConfigContainer()
 		ConfigData(_("%N %V - Help"), ConfigDataType::STR)},
 	{
 		"itemview-title-format",
-		ConfigData(_("%N %V - Article '%T' (%u unread, %t "
-				"total)"),
+		ConfigData(_("%N %V - Article '%T' (%u unread, %t total)"),
 			ConfigDataType::STR)},
 	{
 		"searchresult-title-format",
-		ConfigData(_("%N %V - Search results (%u unread, %t total)"
-				"%?F? matching filter `%F'&?"),
+		ConfigData(_("%N %V - Search results (%u unread, %t total)%?F? matching filter '%F'&?"),
 			ConfigDataType::STR)},
 	{
 		"selectfilter-title-format",
@@ -315,7 +327,8 @@ ConfigContainer::ConfigContainer()
 		ConfigData(_("%N %V - URLs"), ConfigDataType::STR)},
 	{
 		"wrap-scroll",
-		ConfigData("no", ConfigDataType::BOOL)}}
+		ConfigData("no", ConfigDataType::BOOL)},
+}
 {
 }
 
@@ -356,8 +369,10 @@ void ConfigContainer::handle_action(const std::string& action,
 		action,
 		static_cast<unsigned int>(cfgdata.type()));
 
-	if (params.size() < 1) {
+	if (params.size() == 0) {
 		throw ConfigHandlerException(ActionHandlerStatus::TOO_FEW_PARAMS);
+	} else if (params.size() > 1 && !cfgdata.multi_option()) {
+		throw ConfigHandlerException(ActionHandlerStatus::TOO_MANY_PARAMS);
 	}
 
 	switch (cfgdata.type()) {
@@ -456,7 +471,7 @@ void ConfigContainer::toggle(const std::string& key)
 	}
 }
 
-void ConfigContainer::dump_config(std::vector<std::string>& config_output)
+void ConfigContainer::dump_config(std::vector<std::string>& config_output) const
 {
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
 	for (const auto& cfg : config_data) {
