@@ -16,7 +16,6 @@ DialogsFormAction::DialogsFormAction(View* vv,
 	std::string formstr,
 	ConfigContainer* cfg)
 	: FormAction(vv, formstr, cfg)
-	, update_list(true)
 	, dialogs_list("dialogs", FormAction::f,
 		  cfg->get_configvalue_as_int("scrolloff"))
 {
@@ -28,19 +27,14 @@ void DialogsFormAction::init()
 {
 	set_keymap_hints();
 
-	f.run(-3); // compute all widget dimensions
-
-	const unsigned int width = dialogs_list.get_width();
-	std::string title_format = cfg->get_configvalue("dialogs-title-format");
-	FmtStrFormatter fmt;
-	fmt.register_fmt('N', PROGRAM_NAME);
-	fmt.register_fmt('V', utils::program_version());
-	f.set("head", fmt.do_format(title_format, width));
+	recalculate_widget_dimensions();
 }
 
 void DialogsFormAction::prepare()
 {
-	if (update_list) {
+	if (do_redraw) {
+		update_heading();
+
 		ListFormatter listfmt;
 
 		unsigned int i = 1;
@@ -63,16 +57,26 @@ void DialogsFormAction::prepare()
 
 		dialogs_list.stfl_replace_lines(listfmt);
 
-		update_list = false;
+		do_redraw = false;
 	}
 }
 
-KeyMapHintEntry* DialogsFormAction::get_keymap_hint()
+void DialogsFormAction::update_heading()
 {
-	static KeyMapHintEntry hints[] = {{OP_QUIT, _("Close")},
+
+	const unsigned int width = dialogs_list.get_width();
+	const std::string title_format = cfg->get_configvalue("dialogs-title-format");
+	FmtStrFormatter fmt;
+	fmt.register_fmt('N', PROGRAM_NAME);
+	fmt.register_fmt('V', utils::program_version());
+	set_value("head", fmt.do_format(title_format, width));
+}
+
+const std::vector<KeyMapHintEntry>& DialogsFormAction::get_keymap_hint() const
+{
+	static const std::vector<KeyMapHintEntry> hints = {{OP_QUIT, _("Close")},
 		{OP_OPEN, _("Goto Dialog")},
-		{OP_CLOSEDIALOG, _("Close Dialog")},
-		{OP_NIL, nullptr}
+		{OP_CLOSEDIALOG, _("Close Dialog")}
 	};
 	return hints;
 }
@@ -91,9 +95,9 @@ bool DialogsFormAction::process_operation(Operation op,
 		const unsigned int pos = dialogs_list.get_position();
 		if (pos != 0) {
 			v->remove_formaction(pos);
-			update_list = true;
+			do_redraw = true;
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("Error: you can't remove the feed list!"));
 		}
 	}
@@ -139,7 +143,7 @@ void DialogsFormAction::handle_cmdline(const std::string& cmd)
 		if (idx >= 1 && idx <= v->formaction_stack_size()) {
 			dialogs_list.set_position(idx - 1);
 		} else {
-			v->show_error(_("Invalid position!"));
+			v->get_statusline().show_error(_("Invalid position!"));
 		}
 	} else {
 		FormAction::handle_cmdline(cmd);
