@@ -22,6 +22,7 @@ HtmlRenderer::HtmlRenderer(bool raw)
 {
 	tags["a"] = HtmlTag::A;
 	tags["embed"] = HtmlTag::EMBED;
+	tags["iframe"] = HtmlTag::IFRAME;
 	tags["br"] = HtmlTag::BR;
 	tags["pre"] = HtmlTag::PRE;
 	tags["ituneshack"] = HtmlTag::ITUNESHACK;
@@ -29,6 +30,7 @@ HtmlRenderer::HtmlRenderer(bool raw)
 	tags["blockquote"] = HtmlTag::BLOCKQUOTE;
 	tags["aside"] = HtmlTag::BLOCKQUOTE;
 	tags["p"] = HtmlTag::P;
+	tags["div"] = HtmlTag::DIV;
 	tags["h1"] = HtmlTag::H1;
 	tags["h2"] = HtmlTag::H2;
 	tags["h3"] = HtmlTag::H3;
@@ -88,6 +90,16 @@ unsigned int HtmlRenderer::add_link(std::vector<LinkPair>& links,
 	return i;
 }
 
+HtmlTag HtmlRenderer::extract_tag(TagSoupPullParser& parser)
+{
+	std::string tagname = parser.get_text();
+	std::transform(tagname.begin(),
+		tagname.end(),
+		tagname.begin(),
+		::tolower);
+	return tags[tagname];
+}
+
 void HtmlRenderer::render(std::istream& input,
 	std::vector<std::pair<LineType, std::string>>& lines,
 	std::vector<LinkPair>& links,
@@ -97,18 +109,20 @@ void HtmlRenderer::render(std::istream& input,
 	unsigned int video_count = 0;
 	unsigned int audio_count = 0;
 	unsigned int source_count = 0;
+	unsigned int iframe_count = 0;
 	std::string curline;
 	int indent_level = 0;
 	std::vector<HtmlTag> list_elements_stack;
 	bool inside_pre = false;
+	bool pre_just_started = false;
+	size_t pre_consecutive_nl = 0;
 	bool itunes_hack = false;
-	size_t inside_script = 0;
+	bool inside_script = false;
 	size_t inside_style = 0;
 	bool inside_video = false;
 	bool inside_audio = false;
 	std::vector<unsigned int> ol_counts;
 	std::vector<char> ol_types;
-	HtmlTag current_tag;
 	int link_num = -1;
 	std::vector<Table> tables;
 
@@ -121,33 +135,43 @@ void HtmlRenderer::render(std::istream& input,
 	 *   - we then can iterate over all continuous elements, such as start
 	 * tag, close tag, text element, ...
 	 */
-	TagSoupPullParser xpp;
-	xpp.set_input(input);
+	TagSoupPullParser xpp(input);
 
 	for (TagSoupPullParser::Event e = xpp.next();
 		e != TagSoupPullParser::Event::END_DOCUMENT;
 		e = xpp.next()) {
-		std::string tagname;
+		if (inside_script) {
+			// <script> tags can't be nested[1], so we simply ignore all input
+			// while we're looking for the closing tag.
+			//
+			// 1. https://rules.sonarsource.com/html/RSPEC-4645
+
+			switch (e) {
+			case TagSoupPullParser::Event::END_TAG:
+				if (extract_tag(xpp) == HtmlTag::SCRIPT) {
+					inside_script = false;
+				}
+				break;
+
+			default:
+				// Skip everything else.
+				break;
+			}
+
+			// Go on to the next XML node
+			continue;
+		}
+
 		switch (e) {
 		case TagSoupPullParser::Event::START_TAG:
-			tagname = xpp.get_text();
-			std::transform(tagname.begin(),
-				tagname.end(),
-				tagname.begin(),
-				::tolower);
-			current_tag = tags[tagname];
-
-			switch (current_tag) {
+			switch (extract_tag(xpp)) {
 			case HtmlTag::A: {
 				std::string link;
-				try {
-					link = xpp.get_attribute_value("href");
-				} catch (const std::invalid_argument&) {
-					LOG(Level::WARN,
-						"HtmlRenderer::render: found a "
-						"tag "
-						"with no href attribute");
-					link = "";
+				auto href_option = xpp.get_attribute_value("href");
+				if (href_option.has_value()) {
+					link = href_option.value();
+				} else {
+					LOG(Level::WARN, "HtmlRenderer::render: found a tag with no href attribute");
 				}
 				if (link.length() > 0) {
 					link_num = add_link(links,
@@ -179,28 +203,19 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::EMBED: {
 				std::string type;
-				try {
-					type = xpp.get_attribute_value("type");
-				} catch (const std::invalid_argument&) {
-					LOG(Level::WARN,
-						"HtmlRenderer::render: found "
-						"embed "
-						"object without type "
-						"attribute");
-					type = "";
+				auto type_option = xpp.get_attribute_value("type");
+				if (type_option.has_value()) {
+					type = type_option.value();
+				} else {
+					LOG(Level::WARN, "HtmlRenderer::render: found embed object without type attribute");
 				}
 				if (type == "application/x-shockwave-flash") {
 					std::string link;
-					try {
-						link = xpp.get_attribute_value(
-								"src");
-					} catch (const std::invalid_argument&) {
-						LOG(Level::WARN,
-							"HtmlRenderer::render: "
-							"found embed object "
-							"without src "
-							"attribute");
-						link = "";
+					auto link_option = xpp.get_attribute_value("src");
+					if (link_option.has_value()) {
+						link = link_option.value();
+					} else {
+						LOG(Level::WARN, "HtmlRenderer::render: found embed object without src attribute");
 					}
 					if (link.length() > 0) {
 						link_num = add_link(links,
@@ -218,6 +233,46 @@ void HtmlRenderer::render(std::istream& input,
 			}
 			break;
 
+			case HtmlTag::IFRAME: {
+				std::string iframe_url;
+				std::string iframe_title;
+				auto src_option = xpp.get_attribute_value("src");
+				if (src_option.has_value()) {
+					iframe_url = src_option.value();
+				} else {
+					LOG(Level::WARN, "HtmlRenderer::render: found iframe tag without src attribute");
+					iframe_url = "";
+				}
+				auto title_option = xpp.get_attribute_value("title");
+				if (title_option.has_value()) {
+					iframe_title = title_option.value();
+				}
+				if (iframe_url.length() > 0) {
+					add_nonempty_line(curline, tables, lines);
+					if (lines.size() > 0) {
+						std::string::size_type last_line_len =
+							lines[lines.size() - 1]
+							.second.length();
+						if (last_line_len >
+							static_cast<unsigned int>(
+								indent_level * 2)) {
+							add_line("", tables, lines);
+						}
+					}
+					prepare_new_line(curline,
+						tables.size() ? 0 : indent_level);
+
+					iframe_count++;
+					add_media_link(curline, links, url, iframe_url,
+						iframe_title, iframe_count, LinkType::IFRAME);
+
+					add_line(curline, tables, lines);
+					prepare_new_line(curline,
+						tables.size() ? 0 : indent_level);
+				}
+			}
+			break;
+
 			case HtmlTag::BR:
 				add_line(curline, tables, lines);
 				prepare_new_line(curline,
@@ -226,6 +281,7 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::PRE:
 				inside_pre = true;
+				pre_just_started = true;
 				add_nonempty_line(curline, tables, lines);
 				prepare_new_line(curline,
 					tables.size() ? 0 : indent_level);
@@ -236,49 +292,30 @@ void HtmlRenderer::render(std::istream& input,
 				break;
 
 			case HtmlTag::IMG: {
-				std::string imgurl;
-				std::string imgtitle;
-				try {
-					imgurl = xpp.get_attribute_value("src");
-				} catch (const std::invalid_argument&) {
-					LOG(Level::WARN,
-						"HtmlRenderer::render: found "
-						"img "
-						"tag with no src attribute");
-					imgurl = "";
+				std::string img_url;
+				std::string img_label;
+				auto src_option = xpp.get_attribute_value("src");
+				if (src_option.has_value()) {
+					img_url = src_option.value();
+				} else {
+					LOG(Level::WARN, "HtmlRenderer::render: found img tag with no src attribute");
 				}
-				try {
-					imgtitle = xpp.get_attribute_value(
-							"title");
-				} catch (const std::invalid_argument&) {
-					imgtitle = "";
+				// Prefer `alt' over `title'
+				auto alt_option = xpp.get_attribute_value("alt");
+				if (alt_option.has_value()) {
+					img_label = alt_option.value();
 				}
-				if (imgurl.length() > 0) {
-					if (imgurl.substr(0, 5) == "data:") {
-						link_num = add_link(links,
-								"inline image",
-								LinkType::IMG);
-					} else {
-						link_num = add_link(links,
-								utils::censor_url(
-									utils::absolute_url(
-										url,
-										imgurl)),
-								LinkType::IMG);
-					}
-					if (imgtitle != "") {
-						curline.append(strprintf::fmt(
-								"[%s %u: %s]",
-								_("image"),
-								link_num,
-								imgtitle));
-					} else {
-						curline.append(strprintf::fmt(
-								"[%s %u]",
-								_("image"),
-								link_num));
+				if (img_label.empty()) {
+					auto title_option = xpp.get_attribute_value("title");
+					if (title_option.has_value()) {
+						img_label = title_option.value();
 					}
+				}
+				if (!img_url.empty()) {
 					image_count++;
+					add_media_link(curline, links, url,
+						img_url, img_label, image_count,
+						LinkType::IMG);
 				}
 			}
 			break;
@@ -297,7 +334,8 @@ void HtmlRenderer::render(std::istream& input,
 			case HtmlTag::H4:
 			case HtmlTag::H5:
 			case HtmlTag::H6:
-			case HtmlTag::P: {
+			case HtmlTag::P:
+			case HtmlTag::DIV: {
 				add_nonempty_line(curline, tables, lines);
 				if (lines.size() > 0) {
 					std::string::size_type last_line_len =
@@ -319,21 +357,19 @@ void HtmlRenderer::render(std::istream& input,
 				{
 					unsigned int ol_count = 1;
 					std::string ol_count_str;
-					try {
-						ol_count_str =
-							xpp.get_attribute_value(
-								"start");
-					} catch (const std::invalid_argument&) {
+					auto start_option = xpp.get_attribute_value("start");
+					if (start_option.has_value()) {
+						ol_count_str = start_option.value();
+					} else {
 						ol_count_str = "1";
 					}
 					ol_count = utils::to_u(ol_count_str, 1);
 					ol_counts.push_back(ol_count);
 
 					std::string ol_type;
-					try {
-						ol_type =
-							xpp.get_attribute_value(
-								"type");
+					auto type_option = xpp.get_attribute_value("type");
+					if (type_option.has_value()) {
+						ol_type = type_option.value();
 						if (ol_type != "1" &&
 							ol_type != "a" &&
 							ol_type != "A" &&
@@ -341,7 +377,7 @@ void HtmlRenderer::render(std::istream& input,
 							ol_type != "I") {
 							ol_type = "1";
 						}
-					} catch (const std::invalid_argument&) {
+					} else {
 						ol_type = "1";
 					}
 					ol_types.push_back(ol_type[0]);
@@ -441,7 +477,7 @@ void HtmlRenderer::render(std::istream& input,
 					tables.size() ? 0 : indent_level);
 
 				// don't render scripts, ignore current line
-				inside_script++;
+				inside_script = true;
 				break;
 
 			case HtmlTag::STYLE:
@@ -454,11 +490,10 @@ void HtmlRenderer::render(std::istream& input,
 					curline, 0); // no indent in tables
 
 				bool has_border = false;
-				try {
-					std::string b = xpp.get_attribute_value(
-							"border");
-					has_border = (utils::to_u(b, 0) > 0);
-				} catch (const std::invalid_argument&) {
+				auto b = xpp.get_attribute_value("border");
+				if (b.has_value()) {
+					has_border = (utils::to_u(b.value(), 0) > 0);
+				} else {
 					// is ok, no border then
 				}
 				tables.push_back(Table(has_border));
@@ -473,12 +508,10 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::TH: {
 				size_t span = 1;
-				try {
-					span = utils::to_u(
-							xpp.get_attribute_value(
-								"colspan"),
-							1);
-				} catch (const std::invalid_argument&) {
+				auto colspan_option = xpp.get_attribute_value("colspan");
+				if (colspan_option.has_value()) {
+					span = utils::to_u(colspan_option.value(), 1);
+				} else {
 					// is ok, span 1 then
 				}
 				if (!tables.empty()) {
@@ -490,12 +523,10 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::TD: {
 				size_t span = 1;
-				try {
-					span = utils::to_u(
-							xpp.get_attribute_value(
-								"colspan"),
-							1);
-				} catch (const std::invalid_argument&) {
+				auto colspan_option = xpp.get_attribute_value("colspan");
+				if (colspan_option.has_value()) {
+					span = utils::to_u(colspan_option.value(), 1);
+				} else {
 					// is ok, span 1 then
 				}
 				if (!tables.empty()) {
@@ -505,92 +536,117 @@ void HtmlRenderer::render(std::istream& input,
 			}
 
 			case HtmlTag::VIDEO: {
-				std::string videourl;
+				std::string video_url;
 
+				// Decrement the appropriate counter if the
+				// previous media element had no sources
 				if (inside_video && source_count == 0) {
 					video_count--;
 				}
-
 				if (inside_audio && source_count == 0) {
 					audio_count--;
 				}
 
+				// "Pop" the previous media element if it didn't
+				// have a closing tag
 				if (inside_video || inside_audio) {
 					source_count = 0;
 					inside_video = false;
 					inside_audio = false;
 					LOG(Level::WARN,
-						"HtmlRenderer::render media element left unclosed");
+						"HtmlRenderer::render: "
+						"media element left unclosed");
 				}
-
-				try {
-					videourl = xpp.get_attribute_value("src");
-					source_count++;
-				} catch (const std::invalid_argument&) {
-					videourl = "";
-				}
-
-				video_count++;
 				inside_video = true;
-				add_media_link(curline, links, url, videourl,
-					video_count, LinkType::VIDEO);
+
+				// We will decrement this counter later if the
+				// current <video> element contains no sources,
+				// either by its `src' attribute or any child
+				// <source> elements
+				video_count++;
+
+				auto src_option = xpp.get_attribute_value("src");
+				if (src_option.has_value()) {
+					video_url = src_option.value();
+				}
+
+				if (!video_url.empty()) {
+					// Video source retrieved from `src'
+					// attribute
+					source_count++;
+					add_media_link(curline, links, url,
+						video_url, "", video_count,
+						LinkType::VIDEO);
+				}
 			}
 			break;
 
 			case HtmlTag::AUDIO: {
-				std::string audiourl;
+				std::string audio_url;
 
+				// Decrement the appropriate counter if the
+				// previous media element had no sources
 				if (inside_video && source_count == 0) {
 					video_count--;
 				}
-
 				if (inside_audio && source_count == 0) {
 					audio_count--;
 				}
 
+				// "Pop" the previous media element if it didn't
+				// have a closing tag
 				if (inside_video || inside_audio) {
 					source_count = 0;
 					inside_video = false;
 					inside_audio = false;
 					LOG(Level::WARN,
-						"HtmlRenderer::render media element left unclosed");
+						"HtmlRenderer::render: "
+						"media element left unclosed");
 				}
-
-				try {
-					audiourl = xpp.get_attribute_value("src");
-					source_count++;
-				} catch (const std::invalid_argument&) {
-					audiourl = "";
-				}
-
-				audio_count++;
 				inside_audio = true;
-				add_media_link(curline, links, url, audiourl,
-					audio_count, LinkType::AUDIO);
+
+				// We will decrement this counter later if the
+				// current <audio> element contains no sources,
+				// either by its `src' attribute or any child
+				// <source> elements
+				audio_count++;
+
+				auto src_option = xpp.get_attribute_value("src");
+				if (src_option.has_value()) {
+					audio_url = src_option.value();
+				}
+
+				if (!audio_url.empty()) {
+					// Audio source retrieved from `src'
+					// attribute
+					source_count++;
+					add_media_link(curline, links, url,
+						audio_url, "", audio_count,
+						LinkType::AUDIO);
+				}
 			}
 			break;
 
 			case HtmlTag::SOURCE: {
-				std::string sourceurl;
+				std::string source_url;
 
-				try {
-					sourceurl = xpp.get_attribute_value("src");
-					source_count++;
-				} catch (const std::invalid_argument&) {
-					LOG(Level::WARN,
-						"HtmlRenderer::render: found source tag with no src attribute");
-					sourceurl = "";
+				auto src_option = xpp.get_attribute_value("src");
+				if (src_option.has_value()) {
+					source_url = src_option.value();
+				} else {
+					LOG(Level::WARN, "HtmlRenderer::render: found source tag with no src attribute");
 				}
 
-				if (inside_video) {
+				if (inside_video && !source_url.empty()) {
+					source_count++;
 					add_media_link(curline, links, url,
-						sourceurl, video_count,
+						source_url, "", video_count,
 						LinkType::VIDEO);
 				}
-
-				if (inside_audio) {
+				if (inside_audio && !source_url.empty()) {
+					source_count++;
 					add_media_link(curline, links, url,
-						sourceurl, audio_count,
+						source_url, "", audio_count,
 						LinkType::AUDIO);
 				}
 			}
@@ -598,14 +654,7 @@ void HtmlRenderer::render(std::istream& input,
 			break;
 
 		case TagSoupPullParser::Event::END_TAG:
-			tagname = xpp.get_text();
-			std::transform(tagname.begin(),
-				tagname.end(),
-				tagname.begin(),
-				::tolower);
-			current_tag = tags[tagname];
-
-			switch (current_tag) {
+			switch (extract_tag(xpp)) {
 			case HtmlTag::BLOCKQUOTE:
 				--indent_level;
 				if (indent_level < 0) {
@@ -637,7 +686,9 @@ void HtmlRenderer::render(std::istream& input,
 						tables.size() ? 0
 						: indent_level);
 				}
-				list_elements_stack.pop_back();
+				if (!list_elements_stack.empty()) {
+					list_elements_stack.pop_back();
+				}
 				add_nonempty_line(curline, tables, lines);
 				add_line("", tables, lines);
 				prepare_new_line(curline,
@@ -702,16 +753,23 @@ void HtmlRenderer::render(std::istream& input,
 			case HtmlTag::H5:
 			case HtmlTag::H6:
 			case HtmlTag::P:
+			case HtmlTag::DIV:
 				add_nonempty_line(curline, tables, lines);
 				prepare_new_line(curline,
 					tables.size() ? 0 : indent_level);
 				break;
 
 			case HtmlTag::PRE:
-				add_line_softwrappable(curline, lines);
-				prepare_new_line(curline,
-					tables.size() ? 0 : indent_level);
+				if (pre_consecutive_nl > 1) {
+					lines.pop_back();
+				} else if (pre_consecutive_nl == 0) {
+					add_line_softwrappable(curline, lines);
+					prepare_new_line(curline,
+						tables.size() ? 0 : indent_level);
+				}
 				inside_pre = false;
+				pre_just_started = false;
+				pre_consecutive_nl = 0;
 				break;
 
 			case HtmlTag::SUB:
@@ -752,6 +810,7 @@ void HtmlRenderer::render(std::istream& input,
 				break;
 
 			case HtmlTag::EMBED:
+			case HtmlTag::IFRAME:
 			case HtmlTag::BR:
 			case HtmlTag::ITUNESHACK:
 			case HtmlTag::IMG:
@@ -761,12 +820,8 @@ void HtmlRenderer::render(std::istream& input,
 				break;
 
 			case HtmlTag::SCRIPT:
-				// don't render scripts, ignore current line
-				if (inside_script) {
-					inside_script--;
-				}
-				prepare_new_line(curline,
-					tables.size() ? 0 : indent_level);
+				// This line is unreachable, since we handle closing <script>
+				// tags way before this entire `switch`.
 				break;
 
 			case HtmlTag::STYLE:
@@ -895,20 +950,28 @@ void HtmlRenderer::render(std::istream& input,
 				}
 			} else if (inside_pre) {
 				std::vector<std::string> paragraphs =
-					utils::tokenize_nl(text);
+					utils::tokenize_spaced(text, "\r\n");
 				for (const auto& paragraph : paragraphs) {
-					if (paragraph == "\n") {
-						add_line_softwrappable(
-							curline, lines);
-						prepare_new_line(curline,
-							tables.size()
-							? 0
-							: indent_level);
+					if (paragraph.find_first_not_of("\r\n") == std::string::npos) {
+						for (size_t i = 0; i < paragraph.size(); i++) {
+							if (pre_just_started && i == 0) {
+								continue;
+							}
+							add_line_softwrappable(
+								curline, lines);
+							prepare_new_line(curline,
+								tables.size()
+								? 0
+								: indent_level);
+							pre_consecutive_nl++;
+						}
 					} else {
 						curline.append(paragraph);
+						pre_consecutive_nl = 0;
 					}
+					pre_just_started = false;
 				}
-			} else if (inside_script || inside_style || inside_video || inside_audio) {
+			} else if (inside_style || inside_video || inside_audio) {
 				// skip scripts, CSS styles and fallback text for media elements
 			} else {
 				// strip leading whitespace
@@ -934,6 +997,11 @@ void HtmlRenderer::render(std::istream& input,
 		}
 	}
 
+	// remove trailing line break if <pre> is still open
+	if (pre_consecutive_nl > 1) {
+		lines.pop_back();
+	}
+
 	// and the rest
 	add_nonempty_line(curline, tables, lines);
 
@@ -967,19 +1035,30 @@ void HtmlRenderer::render(std::istream& input,
 
 void HtmlRenderer::add_media_link(std::string& curline,
 	std::vector<LinkPair>& links, const std::string& url,
-	const std::string& media_url, unsigned int media_count, LinkType type)
+	const std::string& media_url, const std::string& media_title,
+	unsigned int media_count, LinkType type)
 {
-	if (media_url.empty()) {
-		return;
+	std::string link_url;
+
+	// Display media_url except if it's an inline image
+	if (type == LinkType::IMG && media_url.substr(0, 5) == "data:") {
+		link_url = "inline image";
+	} else {
+		link_url = utils::censor_url(utils::absolute_url(url, media_url));
 	}
 
-	std::string type_str = type2str(type);
-	unsigned int link_num = add_link(links,
-			utils::censor_url(utils::absolute_url(url, media_url)),
-			type);
+	const std::string type_str = type2str(type);
+	const unsigned int link_num = add_link(links, link_url, type);
+	std::string output;
 
-	curline.append(strprintf::fmt("[%s %u (%s #%u)]", _(type_str.c_str()),
-			media_count, _("link"), link_num));
+	if (!media_title.empty()) {
+		output = strprintf::fmt("[%s %u: %s (%s #%u)]", _(type_str.c_str()),
+				media_count, media_title, _("link"), link_num);
+	} else {
+		output = strprintf::fmt("[%s %u (%s #%u)]", _(type_str.c_str()),
+				media_count, _("link"), link_num);
+	}
+	curline.append(output);
 }
 
 std::string HtmlRenderer::render_hr(const unsigned int width)
@@ -1000,6 +1079,8 @@ std::string HtmlRenderer::type2str(LinkType type)
 		return _("image");
 	case LinkType::EMBED:
 		return _("embedded flash");
+	case LinkType::IFRAME:
+		return _("iframe");
 	case LinkType::VIDEO:
 		return _("video");
 	case LinkType::AUDIO:
