@@ -4,13 +4,13 @@
 #include <cerrno>
 #include <cstdio>
 #include <cstring>
-#include <curses.h>
 #include <dirent.h>
 #include <fstream>
 #include <grp.h>
 #include <iomanip>
 #include <iostream>
 #include <libgen.h>
+#include <limits.h>
 #include <ncurses.h>
 #include <pwd.h>
 #include <string.h>
@@ -27,6 +27,8 @@ extern "C" {
 #include "dbexception.h"
 #include "dialogs.h"
 #include "dialogsformaction.h"
+#include "emptyformaction.h"
+#include "empty.h"
 #include "exception.h"
 #include "feedlist.h"
 #include "filebrowser.h"
@@ -63,6 +65,7 @@ View::View(Controller* c)
 	, cfg(0)
 	, keys(0)
 	, current_formaction(0)
+	, status_line(*this)
 	, rxman(c->get_regexmanager())
 	, is_inside_qna(false)
 	, is_inside_cmdline(false)
@@ -106,10 +109,10 @@ void View::set_status(const std::string& msg)
 	std::lock_guard<std::mutex> lock(mtx);
 
 	auto fa = get_current_formaction();
-	if (fa) {
-		Stfl::Form& form = fa->get_form();
-		form.set("msg", msg);
-		form.run(-1);
+	if (fa != nullptr
+		&& std::dynamic_pointer_cast<EmptyFormAction>(fa) == nullptr) {
+		fa->set_value("msg", msg);
+		fa->draw_form();
 	}
 }
 
@@ -118,6 +121,11 @@ void View::show_error(const std::string& msg)
 	set_status(msg);
 }
 
+StatusLine& View::get_statusline()
+{
+	return status_line;
+}
+
 bool View::run_commands(const std::vector<MacroCmd>& commands)
 {
 	for (auto command : commands) {
@@ -126,7 +134,7 @@ bool View::run_commands(const std::vector<MacroCmd>& commands)
 		}
 		std::shared_ptr<FormAction> fa = get_current_formaction();
 		fa->prepare();
-		fa->get_form().run(-1);
+		fa->draw_form();
 		if (!fa->process_op(command.op, true, &command.args)) {
 			// Operation failed, abort
 			return false;
@@ -153,7 +161,7 @@ int View::run()
 
 	if (!run_commands(keys->get_startup_operation_sequence())) {
 		Stfl::reset();
-		std::cerr << _("Error: Failed to execute startup commands") << std::endl;
+		std::cerr << _("Error: failed to execute startup commands") << std::endl;
 		return EXIT_FAILURE;
 	}
 
@@ -169,7 +177,7 @@ int View::run()
 		fa->prepare();
 
 		// we then receive the event and ignore timeouts.
-		const char* event = fa->get_form().run(60000);
+		const std::string event = fa->draw_form_wait_for_event(INT_MAX);
 
 		if (ctrl_c_hit) {
 			ctrl_c_hit = false;
@@ -184,42 +192,28 @@ int View::run()
 			}
 		}
 
-		if (!event || strcmp(event, "TIMEOUT") == 0) {
+		if (event.empty() || event == "TIMEOUT") {
 			continue;
 		}
 
-		if (is_inside_qna) {
-			LOG(Level::DEBUG,
-				"View::run: we're inside QNA input");
-			if (is_inside_cmdline &&
-				strcmp(event, "TAB") == 0) {
-				handle_cmdline_completion(fa);
-				continue;
-			}
-			if (strcmp(event, "^U") == 0) {
-				clear_line(fa);
-				continue;
-			} else if (strcmp(event, "^K") == 0) {
-				clear_eol(fa);
-				continue;
-			} else if (strcmp(event, "^G") == 0) {
-				cancel_input(fa);
-				continue;
-			} else if (strcmp(event, "^W") == 0) {
-				delete_word(fa);
-				continue;
-			}
+		if (event == "RESIZE") {
+			handle_resize();
+			continue;
+		}
+
+		if (handle_qna_event(event, fa)) {
+			continue;
 		}
 
 		LOG(Level::DEBUG, "View::run: event = %s", event);
 
 		if (have_macroprefix) {
 			have_macroprefix = false;
+			status_line.show_message("");
 			LOG(Level::DEBUG,
 				"View::run: running macro `%s'",
 				event);
 			run_commands(keys->get_macro(event));
-			set_status("");
 		} else {
 			const Operation op = keys->get_operation(event, fa->id());
 
@@ -230,7 +224,7 @@ int View::run()
 
 			if (OP_MACROPREFIX == op) {
 				have_macroprefix = true;
-				set_status("macro-");
+				status_line.show_message("macro-");
 			}
 
 			// now we handle the operation to the
@@ -248,6 +242,14 @@ int View::run()
 std::string View::run_modal(std::shared_ptr<FormAction> f,
 	const std::string& value)
 {
+	// Modal dialogs should not allow changing to a different dialog (except by
+	// closing the modal dialog)
+	const std::set<Operation> ignoredOperations = {
+		OP_VIEWDIALOGS,
+		OP_NEXTDIALOG,
+		OP_PREVDIALOG,
+	};
+
 	f->init();
 	unsigned int stacksize = formaction_stack.size();
 
@@ -259,9 +261,18 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 
 		fa->prepare();
 
-		const char* event = fa->get_form().run(1000);
+		const std::string event = fa->draw_form_wait_for_event(INT_MAX);
 		LOG(Level::DEBUG, "View::run: event = %s", event);
-		if (!event || strcmp(event, "TIMEOUT") == 0) {
+		if (event.empty() || event == "TIMEOUT") {
+			continue;
+		}
+
+		if (event == "RESIZE") {
+			handle_resize();
+			continue;
+		}
+
+		if (handle_qna_event(event, fa)) {
 			continue;
 		}
 
@@ -272,6 +283,11 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 			continue;
 		}
 
+		if (ignoredOperations.count(op)) {
+			status_line.show_message(_("Operation ignored in modal dialog"));
+			continue;
+		}
+
 		fa->process_op(op);
 	}
 
@@ -286,17 +302,23 @@ std::string View::get_filename_suggestion(const std::string& s)
 {
 	/*
 	 * With this function, we generate normalized filenames for saving
-	 * articles to files.
+	 * articles to files if the setting `restrict-filename` is enabled.
 	 */
 	std::string retval;
-	for (unsigned int i = 0; i < s.length(); ++i) {
-		if (isalnum(s[i])) {
-			retval.push_back(s[i]);
-		} else if (s[i] == '/' || s[i] == ' ' || s[i] == '\r' ||
-			s[i] == '\n') {
-			retval.push_back('_');
+
+	if (cfg->get_configvalue_as_bool("restrict-filename")) {
+		for (unsigned int i = 0; i < s.length(); ++i) {
+			if (isalnum(s[i])) {
+				retval.push_back(s[i]);
+			} else if (s[i] == '/' || s[i] == ' ' || s[i] == '\r' ||
+				s[i] == '\n') {
+				retval.push_back('_');
+			}
 		}
+	} else {
+		retval = s;
 	}
+
 	if (retval.length() == 0) {
 		retval = "article.txt";
 	} else {
@@ -306,16 +328,13 @@ std::string View::get_filename_suggestion(const std::string& s)
 	return retval;
 }
 
-void View::push_empty_formaction()
+void View::drop_queued_input()
 {
-	formaction_stack.push_back(std::shared_ptr<FormAction>());
-	current_formaction = formaction_stack_size() - 1;
+	flushinp();
 }
 
 void View::open_in_pager(const std::string& filename)
 {
-	formaction_stack.push_back(std::shared_ptr<FormAction>());
-	current_formaction = formaction_stack_size() - 1;
 	std::string cmdline;
 	std::string pager = cfg->get_configvalue("pager");
 	if (pager.find("%f") != std::string::npos) {
@@ -334,32 +353,49 @@ void View::open_in_pager(const std::string& filename)
 		cmdline.append(" ");
 		cmdline.append(filename);
 	}
+	push_empty_formaction();
 	Stfl::reset();
 	utils::run_interactively(cmdline, "View::open_in_pager");
+	drop_queued_input();
 	pop_current_formaction();
 }
 
-nonstd::optional<std::uint8_t> View::open_in_browser(const std::string& url)
+nonstd::optional<std::uint8_t> View::open_in_browser(const std::string& url,
+	const std::string& feedurl, bool interactive)
 {
-	formaction_stack.push_back(std::shared_ptr<FormAction>());
-	current_formaction = formaction_stack_size() - 1;
 	std::string cmdline;
 	const std::string browser = cfg->get_configvalue("browser");
 	const std::string escaped_url = "'" + utils::replace_all(url, "'", "%27") + "'";
-	if (browser.find("%u") != std::string::npos) {
-		cmdline = utils::replace_all(browser, "%u", escaped_url);
+	const std::string escaped_feedurl = "'" + utils::replace_all(feedurl, "'",
+			"%27") + "'";
+
+	if (browser.find("%u") != std::string::npos
+		|| browser.find("%F") != std::string::npos) {
+		cmdline = utils::replace_all(browser, {
+			{"%u", escaped_url},
+			{"%F", escaped_feedurl}
+		});
 	} else {
 		if (browser != "") {
-			cmdline.append(browser);
+			cmdline = browser;
 		} else {
-			cmdline.append("lynx");
+			cmdline = "lynx";
 		}
 		cmdline.append(" " + escaped_url);
 	}
-	Stfl::reset();
-	const auto ret = utils::run_interactively(cmdline, "View::open_in_browser");
-	pop_current_formaction();
-	return ret;
+
+	if (interactive) {
+		push_empty_formaction();
+		Stfl::reset();
+		const auto ret = utils::run_interactively(cmdline, "View::open_in_browser");
+		drop_queued_input();
+		pop_current_formaction();
+		return ret;
+	} else {
+		const std::shared_ptr<AutoDiscardMessage> message =
+			status_line.show_message_until_finished(strprintf::fmt(_("Running browser: %s"), cmdline));
+		return utils::run_non_interactively(cmdline, "View::open_in_browser");
+	}
 }
 
 void View::update_visible_feeds(std::vector<std::shared_ptr<RssFeed>> feeds)
@@ -370,7 +406,7 @@ void View::update_visible_feeds(std::vector<std::shared_ptr<RssFeed>> feeds)
 			feedlist_form->update_visible_feeds(feeds);
 		}
 	} catch (const MatcherException& e) {
-		set_status(strprintf::fmt(
+		status_line.show_message(strprintf::fmt(
 				_("Error: applying the filter failed: %s"), e.what()));
 		LOG(Level::DEBUG,
 			"View::update_visible_feeds: inside catch: %s",
@@ -393,7 +429,7 @@ void View::set_feedlist(std::vector<std::shared_ptr<RssFeed>> feeds)
 			feedlist_form->set_feedlist(feeds);
 		}
 	} catch (const MatcherException& e) {
-		set_status(strprintf::fmt(
+		status_line.show_message(strprintf::fmt(
 				_("Error: applying the filter failed: %s"), e.what()));
 	}
 }
@@ -410,9 +446,8 @@ void View::push_searchresult(std::shared_ptr<RssFeed> feed,
 	LOG(Level::DEBUG, "View::push_searchresult: pushing search result");
 
 	if (feed->total_item_count() > 0) {
-		std::shared_ptr<ItemListFormAction> searchresult(
-			new ItemListFormAction(
-				this, itemlist_str, rsscache, filters, cfg, rxman));
+		auto searchresult = std::make_shared<ItemListFormAction>(
+				this, itemlist_str, rsscache, filters, cfg, rxman);
 		searchresult->set_feed(feed);
 		searchresult->set_show_searchresult(true);
 		searchresult->set_searchphrase(phrase);
@@ -422,7 +457,7 @@ void View::push_searchresult(std::shared_ptr<RssFeed> feed,
 		formaction_stack.push_back(searchresult);
 		current_formaction = formaction_stack_size() - 1;
 	} else {
-		show_error(_("Error: feed contains no items!"));
+		status_line.show_error(_("Error: feed contains no items!"));
 	}
 }
 
@@ -433,12 +468,17 @@ std::shared_ptr<ItemListFormAction> View::push_itemlist(
 
 	feed->purge_deleted_items();
 
-	prepare_query_feed(feed);
+	try {
+		prepare_query_feed(feed);
+	} catch (const MatcherException& e) {
+		const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+		status_line.show_error(msg);
+		return nullptr;
+	}
 
 	if (feed->total_item_count() > 0) {
-		std::shared_ptr<ItemListFormAction> itemlist(
-			new ItemListFormAction(
-				this, itemlist_str, rsscache, filters, cfg, rxman));
+		auto itemlist = std::make_shared<ItemListFormAction>(
+				this, itemlist_str, rsscache, filters, cfg, rxman);
 		itemlist->set_feed(feed);
 		itemlist->set_show_searchresult(false);
 		apply_colors(itemlist);
@@ -448,7 +488,7 @@ std::shared_ptr<ItemListFormAction> View::push_itemlist(
 		current_formaction = formaction_stack_size() - 1;
 		return itemlist;
 	} else {
-		show_error(_("Error: feed contains no items!"));
+		status_line.show_error(_("Error: feed contains no items!"));
 		return nullptr;
 	}
 }
@@ -477,9 +517,8 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 			std::dynamic_pointer_cast<ItemListFormAction,
 			FormAction>(fa);
 		assert(itemlist != nullptr);
-		std::shared_ptr<ItemViewFormAction> itemview(
-			new ItemViewFormAction(
-				this, itemlist, itemview_str, rsscache, cfg, rxman));
+		auto itemview = std::make_shared<ItemViewFormAction>(
+				this, itemlist, itemview_str, rsscache, cfg, rxman);
 		itemview->set_feed(f);
 		itemview->set_guid(guid);
 		itemview->set_parent_formaction(fa);
@@ -502,7 +541,7 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 					item->guid(), true);
 			}
 		} catch (const DbException& e) {
-			show_error(strprintf::fmt(
+			status_line.show_error(strprintf::fmt(
 					_("Error while marking article as read: %s"),
 					e.what()));
 		}
@@ -514,8 +553,8 @@ void View::view_dialogs()
 {
 	auto fa = get_current_formaction();
 	if (fa != nullptr && fa->id() != "dialogs") {
-		std::shared_ptr<DialogsFormAction> dialogs(
-			new DialogsFormAction(this, dialogs_str, cfg));
+		auto dialogs = std::make_shared<DialogsFormAction>(
+				this, dialogs_str, cfg);
 		dialogs->set_parent_formaction(fa);
 		apply_colors(dialogs);
 		dialogs->init();
@@ -524,14 +563,25 @@ void View::view_dialogs()
 	}
 }
 
+void View::push_empty_formaction()
+{
+	auto fa = get_current_formaction();
+
+	auto empty_view = std::make_shared<EmptyFormAction>(
+			this, empty_str, cfg);
+	empty_view->set_parent_formaction(fa);
+	empty_view->init();
+	formaction_stack.push_back(empty_view);
+	current_formaction = formaction_stack_size() - 1;
+}
+
 void View::push_help()
 {
 	auto fa = get_current_formaction();
 
-	std::shared_ptr<HelpFormAction> helpview(
-		new HelpFormAction(this, help_str, cfg));
+	auto helpview = std::make_shared<HelpFormAction>(
+			this, help_str, cfg, fa->id());
 	apply_colors(helpview);
-	helpview->set_context(fa->id());
 	helpview->set_parent_formaction(fa);
 	helpview->init();
 	formaction_stack.push_back(helpview);
@@ -541,8 +591,8 @@ void View::push_help()
 void View::push_urlview(const std::vector<LinkPair>& links,
 	std::shared_ptr<RssFeed>& feed)
 {
-	std::shared_ptr<UrlViewFormAction> urlview(
-		new UrlViewFormAction(this, feed, urlview_str, cfg));
+	auto urlview = std::make_shared<UrlViewFormAction>(
+			this, feed, urlview_str, cfg);
 	apply_colors(urlview);
 	urlview->set_parent_formaction(get_current_formaction());
 	urlview->init();
@@ -551,24 +601,21 @@ void View::push_urlview(const std::vector<LinkPair>& links,
 	current_formaction = formaction_stack_size() - 1;
 }
 
-std::string View::run_filebrowser(const std::string& default_filename,
-	const std::string& dir)
+std::string View::run_filebrowser(const std::string& default_filename)
 {
-	std::shared_ptr<FileBrowserFormAction> filebrowser(
-		new FileBrowserFormAction(this, filebrowser_str, cfg));
+	auto filebrowser = std::make_shared<FileBrowserFormAction>(
+			this, filebrowser_str, cfg);
 	apply_colors(filebrowser);
-	filebrowser->set_dir(dir);
 	filebrowser->set_default_filename(default_filename);
 	filebrowser->set_parent_formaction(get_current_formaction());
 	return run_modal(filebrowser, "filenametext");
 }
 
-std::string View::run_dirbrowser(const std::string& dir)
+std::string View::run_dirbrowser()
 {
-	std::shared_ptr<DirBrowserFormAction> dirbrowser(
-		new DirBrowserFormAction(this, filebrowser_str, cfg));
+	auto dirbrowser = std::make_shared<DirBrowserFormAction>(
+			this, filebrowser_str, cfg);
 	apply_colors(dirbrowser);
-	dirbrowser->set_dir(dir);
 	dirbrowser->set_parent_formaction(get_current_formaction());
 	return run_modal(dirbrowser, "filenametext");
 }
@@ -576,11 +623,11 @@ std::string View::run_dirbrowser(const std::string& dir)
 std::string View::select_tag()
 {
 	if (tags.size() == 0) {
-		show_error(_("No tags defined."));
+		status_line.show_error(_("No tags defined."));
 		return "";
 	}
-	std::shared_ptr<SelectFormAction> selecttag(
-		new SelectFormAction(this, selecttag_str, cfg));
+	auto selecttag = std::make_shared<SelectFormAction>(
+			this, selecttag_str, cfg);
 	selecttag->set_type(SelectFormAction::SelectionType::TAG);
 	apply_colors(selecttag);
 	selecttag->set_parent_formaction(get_current_formaction());
@@ -591,8 +638,8 @@ std::string View::select_tag()
 
 std::string View::select_filter(const std::vector<FilterNameExprPair>& filters)
 {
-	std::shared_ptr<SelectFormAction> selecttag(
-		new SelectFormAction(this, selecttag_str, cfg));
+	auto selecttag = std::make_shared<SelectFormAction>(
+			this, selecttag_str, cfg);
 	selecttag->set_type(SelectFormAction::SelectionType::FILTER);
 	apply_colors(selecttag);
 	selecttag->set_parent_formaction(get_current_formaction());
@@ -606,19 +653,19 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 	LOG(Level::DEBUG, "View::confirm: charset = %s", charset);
 
 	std::shared_ptr<FormAction> f = get_current_formaction();
-	formaction_stack.push_back(std::shared_ptr<FormAction>());
-	current_formaction = formaction_stack_size() - 1;
-	f->get_form().set("msg", prompt);
+	// Push empty formaction so our "msg" is not overwritten
+	push_empty_formaction();
+	f->set_value("msg", prompt);
 
 	char result = 0;
 
 	do {
-		const char* event = f->get_form().run(0);
+		const std::string event = f->draw_form_wait_for_event(0);
 		LOG(Level::DEBUG, "View::confirm: event = %s", event);
-		if (!event) {
+		if (event.empty()) {
 			continue;
 		}
-		if (strcmp(event, "ESC") == 0 || strcmp(event, "ENTER") == 0) {
+		if (event == "ESC" || event == "ENTER") {
 			result = 0;
 			LOG(Level::DEBUG,
 				"View::confirm: user pressed ESC or ENTER, we "
@@ -632,8 +679,8 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 			result);
 	} while (!result || strchr(charset.c_str(), result) == nullptr);
 
-	f->get_form().set("msg", "");
-	f->get_form().run(-1);
+	f->set_value("msg", "");
+	f->draw_form();
 
 	pop_current_formaction();
 
@@ -654,6 +701,7 @@ void View::notify_itemlist_change(std::shared_ptr<RssFeed> feed)
 					f->rssurl() == feed->rssurl()) {
 					itemlist->set_feed(feed);
 					itemlist->set_redraw(true);
+					itemlist->invalidate_list();
 				}
 			}
 		}
@@ -671,7 +719,13 @@ bool View::get_random_unread(ItemListFormAction& itemlist,
 		LOG(Level::DEBUG,
 			"View::get_random_unread: found feed with unread "
 			"articles");
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -707,12 +761,18 @@ bool View::get_previous_unread(ItemListFormAction& itemlist,
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG,
 			"View::get_previous_unread: goto-next-feed = false");
-		show_error(_("No unread items."));
+		status_line.show_error(_("No unread items."));
 	} else if (feedlist_form->jump_to_previous_unread_feed(feedpos)) {
 		LOG(Level::DEBUG,
 			"View::get_previous_unread: found feed with unread "
 			"articles");
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -732,7 +792,13 @@ bool View::get_next_unread_feed(ItemListFormAction& itemlist)
 {
 	unsigned int feedpos;
 	if (feedlist_form->jump_to_next_unread_feed(feedpos)) {
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -745,7 +811,13 @@ bool View::get_prev_unread_feed(ItemListFormAction& itemlist)
 {
 	unsigned int feedpos;
 	if (feedlist_form->jump_to_previous_unread_feed(feedpos)) {
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -772,12 +844,18 @@ bool View::get_next_unread(ItemListFormAction& itemlist,
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG,
 			"View::get_next_unread: goto-next-feed = false");
-		show_error(_("No unread items."));
+		status_line.show_error(_("No unread items."));
 	} else if (feedlist_form->jump_to_next_unread_feed(feedpos)) {
 		LOG(Level::DEBUG,
 			"View::get_next_unread: found feed with unread "
 			"articles");
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -807,10 +885,16 @@ bool View::get_previous(ItemListFormAction& itemlist,
 		return true;
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG, "View::get_previous: goto-next-feed = false");
-		show_error(_("Already on first item."));
+		status_line.show_error(_("Already on first item."));
 	} else if (feedlist_form->jump_to_previous_feed(feedpos)) {
 		LOG(Level::DEBUG, "View::get_previous: previous feed");
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -839,10 +923,16 @@ bool View::get_next(ItemListFormAction& itemlist, ItemViewFormAction* itemview)
 		return true;
 	} else if (cfg->get_configvalue_as_bool("goto-next-feed") == false) {
 		LOG(Level::DEBUG, "View::get_next: goto-next-feed = false");
-		show_error(_("Already on last item."));
+		status_line.show_error(_("Already on last item."));
 	} else if (feedlist_form->jump_to_next_feed(feedpos)) {
 		LOG(Level::DEBUG, "View::get_next: next feed");
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -862,7 +952,13 @@ bool View::get_next_feed(ItemListFormAction& itemlist)
 {
 	unsigned int feedpos;
 	if (feedlist_form->jump_to_next_feed(feedpos)) {
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -875,7 +971,13 @@ bool View::get_prev_feed(ItemListFormAction& itemlist)
 {
 	unsigned int feedpos;
 	if (feedlist_form->jump_to_previous_feed(feedpos)) {
-		prepare_query_feed(feedlist_form->get_feed());
+		try {
+			prepare_query_feed(feedlist_form->get_feed());
+		} catch (const MatcherException& e) {
+			const auto msg = strprintf::fmt(_("Error: couldn't prepare query feed: %s"), e.what());
+			status_line.show_error(msg);
+			return false;
+		}
 		itemlist.set_feed(feedlist_form->get_feed());
 		itemlist.set_pos(feedpos);
 		itemlist.init();
@@ -891,21 +993,22 @@ void View::prepare_query_feed(std::shared_ptr<RssFeed> feed)
 			"View::prepare_query_feed: %s",
 			feed->rssurl());
 
-		set_status(_("Updating query feed..."));
+		const std::shared_ptr<AutoDiscardMessage> message =
+			status_line.show_message_until_finished(_("Updating query feed..."));
 		feed->update_items(ctrl->get_feedcontainer()->get_all_feeds());
 		feed->sort(cfg->get_article_sort_strategy());
 		notify_itemlist_change(feed);
-		set_status("");
 	}
 }
 
 void View::force_redraw()
 {
 	std::shared_ptr<FormAction> fa = get_current_formaction();
-	if (fa != nullptr) {
+	if (fa != nullptr
+		&& std::dynamic_pointer_cast<EmptyFormAction>(fa) == nullptr) {
 		fa->set_redraw(true);
 		fa->prepare();
-		fa->get_form().run(-1);
+		fa->draw_form();
 	}
 }
 
@@ -918,7 +1021,7 @@ void View::pop_current_formaction()
 	}
 	formaction_stack.erase(it);
 	if (f == nullptr) {
-		// XXX TODO this is not correct... we'd need to return to the previous one, but nullptr formactions have no parent
+		// TODO this is not correct... we'd need to return to the previous one, but nullptr formactions have no parent
 		current_formaction = formaction_stack_size() -
 			1;
 	} else if (formaction_stack.size() > 0) {
@@ -940,11 +1043,15 @@ void View::pop_current_formaction()
 			}
 			i++;
 		}
-		std::shared_ptr<FormAction> f = get_current_formaction();
-		if (f) {
-			f->set_redraw(true);
-			f->get_form().set("msg", "");
-			f->recalculate_form();
+
+		// Skip cleanup steps when returning from a transient EmptyFormAction
+		if (std::dynamic_pointer_cast<EmptyFormAction>(f) == nullptr) {
+			std::shared_ptr<FormAction> fa = get_current_formaction();
+			if (fa) {
+				fa->set_redraw(true);
+				fa->set_value("msg", "");
+				fa->recalculate_widget_dimensions();
+			}
 		}
 	}
 }
@@ -991,7 +1098,11 @@ void View::apply_colors(std::shared_ptr<FormAction> fa)
 {
 	LOG(Level::DEBUG, "View::apply_colors: fa = %s", fa->id());
 
-	colorman.apply_colors(fa->get_form());
+	const auto stfl_value_setter = [&](const std::string& name,
+	const std::string& value) {
+		fa->set_value(name, value);
+	};
+	colorman.apply_colors(stfl_value_setter);
 }
 
 void View::feedlist_mark_pos_if_visible(unsigned int pos)
@@ -1051,18 +1162,18 @@ void View::inside_cmdline(bool f)
 
 void View::clear_line(std::shared_ptr<FormAction> fa)
 {
-	fa->get_form().set("qna_value", "");
-	fa->get_form().set("qna_value_pos", "0");
+	fa->set_value("qna_value", "");
+	fa->set_value("qna_value_pos", "0");
 	LOG(Level::DEBUG, "View::clear_line: cleared line");
 }
 
 void View::clear_eol(std::shared_ptr<FormAction> fa)
 {
-	unsigned int pos = utils::to_u(fa->get_form().get("qna_value_pos"), 0);
-	std::string val = fa->get_form().get("qna_value");
+	unsigned int pos = utils::to_u(fa->get_value("qna_value_pos"), 0);
+	std::string val = fa->get_value("qna_value");
 	val.erase(pos, val.length());
-	fa->get_form().set("qna_value", val);
-	fa->get_form().set("qna_value_pos", std::to_string(val.length()));
+	fa->set_value("qna_value", val);
+	fa->set_value("qna_value_pos", std::to_string(val.length()));
 	LOG(Level::DEBUG, "View::clear_eol: cleared to end of line");
 }
 
@@ -1075,8 +1186,8 @@ void View::cancel_input(std::shared_ptr<FormAction> fa)
 void View::delete_word(std::shared_ptr<FormAction> fa)
 {
 	std::string::size_type curpos =
-		utils::to_u(fa->get_form().get("qna_value_pos"), 0);
-	std::string val = fa->get_form().get("qna_value");
+		utils::to_u(fa->get_value("qna_value_pos"), 0);
+	std::string val = fa->get_value("qna_value");
 	std::string::size_type firstpos = curpos;
 	LOG(Level::DEBUG, "View::delete_word: before val = %s", val);
 	if (firstpos >= val.length() || ::isspace(val[firstpos])) {
@@ -1095,13 +1206,51 @@ void View::delete_word(std::shared_ptr<FormAction> fa)
 	}
 	val.erase(firstpos, curpos - firstpos);
 	LOG(Level::DEBUG, "View::delete_word: after val = %s", val);
-	fa->get_form().set("qna_value", val);
-	fa->get_form().set("qna_value_pos", std::to_string(firstpos));
+	fa->set_value("qna_value", val);
+	fa->set_value("qna_value_pos", std::to_string(firstpos));
+}
+
+bool View::handle_qna_event(const std::string& event,
+	std::shared_ptr<FormAction> fa)
+{
+	if (is_inside_qna) {
+		LOG(Level::DEBUG,
+			"View::handle_qna_event: we're inside QNA input");
+		if (is_inside_cmdline && event == "TAB") {
+			handle_cmdline_completion(fa);
+			return true;
+		}
+		if (event == "^U") {
+			clear_line(fa);
+			return true;
+		} else if (event == "^K") {
+			clear_eol(fa);
+			return true;
+		} else if (event == "^G") {
+			cancel_input(fa);
+			return true;
+		} else if (event == "^W") {
+			delete_word(fa);
+			return true;
+		}
+	}
+	return false;
+}
+
+void View::handle_resize()
+{
+	for (const auto& form : formaction_stack) {
+		if (form != nullptr) {
+			// Recalculate width and height of stfl widgets
+			form->recalculate_widget_dimensions();
+			form->set_redraw(true);
+		}
+	}
 }
 
 void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 {
-	std::string fragment = fa->get_form().get("qna_value");
+	std::string fragment = fa->get_value("qna_value");
 	if (fragment != last_fragment || fragment == "") {
 		last_fragment = fragment;
 		suggestions = fa->get_suggestions(fragment);
@@ -1126,33 +1275,11 @@ void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 		suggestion = suggestions[(tab_count - 1) % suggestions.size()];
 		break;
 	}
-	fa->get_form().set("qna_value", suggestion);
-	fa->get_form().set(
-		"qna_value_pos", std::to_string(suggestion.length()));
+	fa->set_value("qna_value", suggestion);
+	fa->set_value("qna_value_pos", std::to_string(suggestion.length()));
 	last_fragment = suggestion;
 }
 
-void View::dump_current_form()
-{
-	std::string formtext =
-		formaction_stack[current_formaction]->get_form().dump(
-			"", "", 0);
-	time_t t = time(nullptr);
-	const auto fnbuf = utils::mt_strf_localtime(
-			"dumpform-%Y%m%d-%H%M%S.stfl",
-			t);
-	std::fstream f(fnbuf, std::ios_base::out);
-	if (!f.is_open()) {
-		show_error(strprintf::fmt("Error: couldn't open file %s: %s",
-				fnbuf,
-				strerror(errno)));
-		return;
-	}
-	f << formtext;
-	f.close();
-	set_status(strprintf::fmt("Dumped current form to file %s", fnbuf));
-}
-
 void View::ctrl_c_action(int /* sig */)
 {
 	LOG(Level::DEBUG, "caught SIGINT");
