@@ -250,6 +250,44 @@ TEST_CASE(
 	}
 }
 
+TEST_CASE("Supports combined short options", "[CliArgsParser]")
+{
+	const std::string filename("cache.db");
+
+	TestHelpers::Opts opts = {"newsboat", "-vc", filename};
+	CliArgsParser args(opts.argc(), opts.argv());
+
+	REQUIRE(args.cache_file() == filename);
+	REQUIRE(args.lock_file() == filename + ".lock");
+	REQUIRE(args.using_nonstandard_configs());
+	REQUIRE(args.show_version() == 1);
+}
+
+TEST_CASE("Supports combined short option and value", "[CliArgsParser]")
+{
+	const std::string filename("cache.db");
+
+	TestHelpers::Opts opts = {"newsboat", "-c" + filename};
+	CliArgsParser args(opts.argc(), opts.argv());
+
+	REQUIRE(args.cache_file() == filename);
+	REQUIRE(args.lock_file() == filename + ".lock");
+	REQUIRE(args.using_nonstandard_configs());
+}
+
+TEST_CASE("Supports `=` between short option and value",
+	"[CliArgsParser]")
+{
+	const std::string filename("cache.db");
+
+	TestHelpers::Opts opts = {"newsboat", "-c=" + filename};
+	CliArgsParser args(opts.argc(), opts.argv());
+
+	REQUIRE(args.cache_file() == filename);
+	REQUIRE(args.lock_file() == filename + ".lock");
+	REQUIRE(args.using_nonstandard_configs());
+}
+
 TEST_CASE("Resolves tilde to homedir in -c/--cache-file", "[CliArgsParser]")
 {
 	TestHelpers::TempDir tmp;
@@ -342,6 +380,19 @@ TEST_CASE("Sets `do_vacuum` if -X/--vacuum is provided", "[CliArgsParser]")
 	}
 }
 
+TEST_CASE("Sets `do_cleanup` if --cleanup is provided", "[CliArgsParser]")
+{
+	auto check = [](TestHelpers::Opts opts) {
+		CliArgsParser args(opts.argc(), opts.argv());
+
+		REQUIRE(args.do_cleanup());
+	};
+
+	SECTION("--cleanup") {
+		check({"newsboat", "--cleanup"});
+	}
+}
+
 TEST_CASE("Increases `show_version` with each -v/-V/--version provided",
 	"[CliArgsParser]")
 {
@@ -402,8 +453,7 @@ TEST_CASE("Sets `execute_cmds` if -x/--execute is provided", "[CliArgsParser]")
 	auto check = [](TestHelpers::Opts opts) {
 		CliArgsParser args(opts.argc(), opts.argv());
 
-		REQUIRE_FALSE(args.cmds_to_execute() == nonstd::nullopt);
-		REQUIRE_FALSE(args.cmds_to_execute().value().empty());
+		REQUIRE_FALSE(args.cmds_to_execute().empty());
 	};
 
 	SECTION("-x") {
@@ -703,3 +753,26 @@ TEST_CASE("Sets `program_name` to the first string of the options list",
 	check({"/usr/local/bin/app-with-a-path"},
 		"/usr/local/bin/app-with-a-path");
 }
+
+TEST_CASE("Test should fail on equal sign with multiple values",
+	"[CliArgsParser]")
+{
+	auto check = [](TestHelpers::Opts opts, const std::vector<std::string>& cmds) {
+		CliArgsParser args(opts.argc(), opts.argv());
+		REQUIRE(args.should_print_usage());
+		REQUIRE(args.cmds_to_execute() != cmds);
+	};
+
+	check({"newsboat", "-x=reload", "print-unread" }, { "reload", "print-unread" });
+	check({"newsboat", "--execute=reload", "print-unread" }, { "reload", "print-unread" });
+
+}
+
+TEST_CASE("Supports combined short options where last has equal sign",
+	"[CliArgsParser]")
+{
+	TestHelpers::Opts opts = {"newsboat", "-rx=reload"};
+	CliArgsParser args(opts.argc(), opts.argv());
+	std::vector<std::string> commands{"reload"};
+	REQUIRE(args.cmds_to_execute() == commands);
+}
\ No newline at end of file
