@@ -17,9 +17,8 @@ TEST_CASE("Tagsoup pull parser turns document into a stream of events",
 		"<xxx foo=bar baz=\"qu ux\" hi='ho ho ho'></xxx>"
 		"</test>");
 
-	TagSoupPullParser xpp;
+	TagSoupPullParser xpp(input_stream);
 	TagSoupPullParser::Event e;
-	xpp.set_input(input_stream);
 
 	e = xpp.get_event_type();
 	REQUIRE(e == TagSoupPullParser::Event::START_DOCUMENT);
@@ -31,8 +30,8 @@ TEST_CASE("Tagsoup pull parser turns document into a stream of events",
 	e = xpp.next();
 	REQUIRE(e == TagSoupPullParser::Event::START_TAG);
 	REQUIRE(xpp.get_text() == "foo");
-	REQUIRE(xpp.get_attribute_value("quux") == "asdf");
-	REQUIRE(xpp.get_attribute_value("bar") == "qqq");
+	REQUIRE(xpp.get_attribute_value("quux").value() == "asdf");
+	REQUIRE(xpp.get_attribute_value("bar").value() == "qqq");
 
 	e = xpp.next();
 	REQUIRE(e == TagSoupPullParser::Event::TEXT);
@@ -61,9 +60,9 @@ TEST_CASE("Tagsoup pull parser turns document into a stream of events",
 	e = xpp.next();
 	REQUIRE(e == TagSoupPullParser::Event::START_TAG);
 	REQUIRE(xpp.get_text() == "xxx");
-	REQUIRE(xpp.get_attribute_value("foo") == "bar");
-	REQUIRE(xpp.get_attribute_value("baz") == "qu ux");
-	REQUIRE(xpp.get_attribute_value("hi") == "ho ho ho");
+	REQUIRE(xpp.get_attribute_value("foo").value() == "bar");
+	REQUIRE(xpp.get_attribute_value("baz").value() == "qu ux");
+	REQUIRE(xpp.get_attribute_value("hi").value() == "ho ho ho");
 
 	e = xpp.next();
 	REQUIRE(e == TagSoupPullParser::Event::END_TAG);
@@ -83,7 +82,6 @@ TEST_CASE("Tagsoup pull parser turns document into a stream of events",
 TEST_CASE("<br>, <br/> and <br /> behave the same way", "[TagSoupPullParser]")
 {
 	std::istringstream input_stream;
-	TagSoupPullParser Parser;
 	TagSoupPullParser::Event event;
 
 	for (auto input : {
@@ -91,7 +89,7 @@ TEST_CASE("<br>, <br/> and <br /> behave the same way", "[TagSoupPullParser]")
 		}) {
 		SECTION(input) {
 			input_stream.str(input);
-			Parser.set_input(input_stream);
+			TagSoupPullParser Parser(input_stream);
 
 			event = Parser.get_event_type();
 			REQUIRE(event ==
@@ -118,9 +116,8 @@ TEST_CASE("Tagsoup pull parser emits whitespace as is",
 		"</pre>"
 		"</test>");
 
-	TagSoupPullParser xpp;
+	TagSoupPullParser xpp(input_stream);
 	TagSoupPullParser::Event e;
-	xpp.set_input(input_stream);
 
 	e = xpp.get_event_type();
 	REQUIRE(e == TagSoupPullParser::Event::START_DOCUMENT);
@@ -167,3 +164,61 @@ TEST_CASE("Tagsoup pull parser emits whitespace as is",
 	e = xpp.next();
 	REQUIRE(e == TagSoupPullParser::Event::END_DOCUMENT);
 }
+
+TEST_CASE("TagSoupPullParser can decode HTML entities", "[TagSoupPullParser]")
+{
+	SECTION("Numbered entites") {
+		SECTION("Decimal") {
+			// 133 is used in its non-standard meaning here, designating a horizontal ellipsis
+			std::istringstream input_stream("&#020;&#42;&#189;&#133;&#963;");
+
+			TagSoupPullParser xpp(input_stream);
+			TagSoupPullParser::Event e;
+
+			e = xpp.get_event_type();
+			REQUIRE(e == TagSoupPullParser::Event::START_DOCUMENT);
+
+			e = xpp.next();
+			REQUIRE(e == TagSoupPullParser::Event::TEXT);
+			REQUIRE(xpp.get_text() == "\x14*½…σ");
+
+			e = xpp.next();
+			REQUIRE(e == TagSoupPullParser::Event::END_DOCUMENT);
+		}
+
+		SECTION("Hexadecimal") {
+			// x97 is used in its non-standard meaning here, designating an mdash
+			std::istringstream input_stream("&#x97;&#x20;&#x048;&#x0069;");
+
+			TagSoupPullParser xpp(input_stream);
+			TagSoupPullParser::Event e;
+
+			e = xpp.get_event_type();
+			REQUIRE(e == TagSoupPullParser::Event::START_DOCUMENT);
+
+			e = xpp.next();
+			REQUIRE(e == TagSoupPullParser::Event::TEXT);
+			REQUIRE(xpp.get_text() == "— Hi");
+
+			e = xpp.next();
+			REQUIRE(e == TagSoupPullParser::Event::END_DOCUMENT);
+		}
+	}
+
+	SECTION("Named entities") {
+		std::istringstream input_stream("&sigma;&trade;");
+
+		TagSoupPullParser xpp(input_stream);
+		TagSoupPullParser::Event e;
+
+		e = xpp.get_event_type();
+		REQUIRE(e == TagSoupPullParser::Event::START_DOCUMENT);
+
+		e = xpp.next();
+		REQUIRE(e == TagSoupPullParser::Event::TEXT);
+		REQUIRE(xpp.get_text() == "σ™");
+
+		e = xpp.next();
+		REQUIRE(e == TagSoupPullParser::Event::END_DOCUMENT);
+	}
+}
